!==========================================================================================================!
! Main code for sea surface albedo computation                                                             !  
!                                                                                                          !
! Auxiliary parameters from module 'rrsw_ocaux':                                                           !
! 'wl': wavelength covers all RRTMG_SW channels except the last channel.                                   !
! 'refid': refractive index of sea water                                                                   !
! 'abs_sw': absorption coefficients of pure sea water; unit: m^(-1)                                        !
! 'sca_sw': scattering coefficients of pure sea water; unit: m^(-1)                                        !
! 'stand_solar': standard irradiance solar spectra; unit: W/m^2/nm                                         !
!                                                                                                          !
! specific absorption coefficients of phytoplankton: applicable: 300 - 398nm; unit: m ** 2 / mg            !
! power-law: a*p(wl) = A(wl) * chl_con ** (- B(wl))                                                        !
! 'specific_A': A(wl); 'specific_B': B(wl)                                                                 !
!                                                                                                          !
! specific absorption coefficients of phytoplankton: applicable: 400 - 700nm; unit: m ** 2 / mg            !
! size-fraction phytoplankton absoption coefficients                                                       !
! three size: pico- (2nm<), nano- (2-20nm), micro-phytoplankton (>20nm)                                    !
! 'specific_ap'; 'specific_an'; 'specific_am';                                                             !
!                                                                                                          !
!                                                                                                          !
!==========================================================================================================!
!                                                                                                          !
! Date: Sep.,2020                                                                                          !
! Author: Jian Wei.       Email: anser@tamu.edu      or     anserwei@gmail.com                             !
! Atmospheric, Oceanic Optics and Remote Sensing Lab, Texas A&M University                                 !
!                                                                                                          !
!==========================================================================================================!


module rrtmg_sw_ssal
    use rrsw_ocaux
    implicit none

    !  declare 'chan_pos' in modoule 'ocaux'
    ! 'chan_pos': RRRTMG channel position: each column indicates one band; 1st row & 2nd row: wavelength range. 
    !
    !Band #   Wavenumber (cm-1)   (nm)           1050 - 96 mb      96 - 0.01 mb       chan_pos (see rrsw_ocaux.f90)
    !16        2600-3250    (3846.1-3076.9)         H2O,CH4             CH4                chan_pos(2,13) - chan_pos(1,13)
    !17        3250-4000    (3076.9-2500.0)         H2O,CO2           H2O,CO2              chan_pos(2,12) - chan_pos(1,12)
    !18        4000-4650    (2500.0-2150.5)         H2O,CH4             CH4                chan_pos(2,11) - chan_pos(1,11)
    !18        4650-5150    (2150.5-1941.8)         H2O,CO2             CO2                chan_pos(2,10) - chan_pos(1,10)
    !20        5150-6150    (1941.8-1626.0)         H2O,CH4*          H2O,CH4*              chan_pos(2,9) - chan_pos(1,9)
    !21        6150-7700    (1626.0-1298.7)         H2O,CO2           H2O,CO2               chan_pos(2,8) - chan_pos(1,8)
    !22        7700-8050    (1298.7-1242.4)         H2O,O2              O2                  chan_pos(2,7) - chan_pos(1,7)
    !23        8050-12850   (1242.4- 778.2)           H2O             nothing               chan_pos(2,6) - chan_pos(1,6)
    !24       12850-16000   (778.2 - 625.0)        H2O,O2,O3*          O2,O3*               chan_pos(2,5) - chan_pos(1,5)
    !25       16000-22650   (625.0 - 441.5)         H2O,O3*             O3*                 chan_pos(2,4) - chan_pos(1,4)
    !26       22650-29000   (441.5 - 344.8)         nothing           nothing               chan_pos(2,3) - chan_pos(1,3)
    !20       29000-38000   (344.8 - 263.1)           O3                O3                  chan_pos(2,2) - chan_pos(1,2)
    !28       38000-50000   (263.1 - 200.0)          O3,O2            O3,O2                 chan_pos(2,1) - chan_pos(1,1)
    !29         820-2600   - no applicable -          H2O               CO2                   - - - not applicable - - - 


!====================================== subroutines -- broadband albedo ========================================
!

    contains

    subroutine spctral_em(sza, chl_con, wspd, wspd_dir, chan_pos_start, chan_pos_end, bd_dhr_semiss, bd_bhr_semiss)
        use rrsw_ocaux
        implicit none
        real(8), intent(in) :: sza, chl_con, wspd, wspd_dir
        integer, intent(in) :: chan_pos_start, chan_pos_end
        real(8), intent(out) :: bd_dhr_semiss, bd_bhr_semiss

        ! subroutines output from 'whitecaps_al'; 'waterleaving_al';'sgbsky_al'
        real(8) :: wc_refl, wl_refl, dhr, bhr       
       
        ! varibles used for computations in this subroutine
        integer :: N, i, j
        real(8) :: integral_n, final_numerator, integral_d, final_denominator, &
                   integral_nb, final_numerator_b, integral_db, final_denominator_b, &
                   diffuse_sza
        real(8), parameter :: pi = acos(- 1.0), h = 4.0        
        real(8), dimension(913) :: numerator, numerator_b
	real(8) :: bd_dhr, bd_bhr  
        
        interface
            subroutine blacksky_al(loc, sza, chl_con, wspd, wspd_dir, dhr)
                use rrsw_ocaux
                implicit none
                ! declare variables in & out
                integer, intent(in) :: loc             ! position of wavelength
	        real(8), intent(in) :: sza             ! solar zenith angle in degree
	        real(8), intent(in) :: chl_con         ! chlorophyll concentration (mg/m-3)
	        real(8), intent(in) :: wspd            ! wind speed (m/s)
	        real(8), intent(in) :: wspd_dir        ! wind direction in degree [0.0 ~ 360.0)
	        real(8), intent(out) :: dhr            ! blacksky albedo

                ! declare varibles for 'whitecaps'
	        real(8) :: temp, wc_refl               ! whitecap reflectance
	        real(8), parameter :: a0=0.47, a1 = - 1.62, a2 = - 8.66, a3 = 31.81       ! constents

                ! declare varibles for water leaving, 
	        real(8), parameter :: p_m = 0.043      ! constents
                ! parameters for calculating absorption and backscattering
	        real(8) :: ab_cdom440, ab_cdom, ab_nap440, ab_nap, ab_phy, C_pn, C_p, C_n, C_m, abs_tots
	        real(8) :: lamda0, Cm12, S12, bp12wl0, bp3wl0, bpkwl0, gama12, gama3, gamak, bbp, bb_tots
                ! variables for subsurface remote sensing reflectance
	        real(8), parameter :: pi = acos(- 1.0)
	        real(8) :: vza(91) = (/(iv,iv=0,90,1)/)
	        real(8) :: aza(361) = (/(iv,iv=0,360,1)/)
	        real(8) :: ui, sin_sza_w, Kd, ita_bb, bb_a, V, e1, e2, e3, e4, e5, e6, e7, e8, e9, &
                           e10, e11, e12, e13, e14, e15, e16, e17, e18, H, P3u, sza_w, Mdp, mds1, &
                           mds2, mds3, mds4, mds5, mds6, mds7, mds8, Mds, audp, audp_infi, fa1, &
                           fa2, fa3, fa4, fa5, bb_tots_ratio
	        real(8), dimension(91) :: uv
	        real(8), dimension(361) :: uaz
	        real(8) :: uscat_wl, angscat_wl, pf_bp, pf_bw, faup, vsf, rrs, wl_refl
	        real(8) :: refid_rwa, refid_raw, sintheta_wp, Rv, Rp, Rvp, Rwlr_tem

                ! declare variables for 'sun glint'
	        real(8) :: geda, etax, A_ui, shadow_x, wspd_dir_m, geda_x, geda_y
	        real(8) :: C21, C03, C40, C22, C04
	        real(8), dimension(91) :: etay, A_uv, shadow_y, shadow_c
	        real(8), dimension(91, 361) :: uscat, angscat, unorm, locin_ang
	        real(8), dimension(91, 361) :: Zx, Zy, Zx_n, Zy_n, ksai_x, Ksai_y
	        real(8), dimension(91, 361) :: clocin, slocin, Rvu, Rpu, Rvpu
	        real(8), dimension(91, 361) :: GCs, PDF, denomi
	        real(8), dimension(91, 361) :: sg_refl
                integer :: iuv, iuaz
       
                ! decalare for integration
	        real(8) :: wc_fra
	        real(8), dimension(91, 361):: BRDF, infunc
	        real(8)       :: a, b, c, d, hx, hy, integral, integral_2D, dhr_temp
                integer                :: i, j, NX, NY, a_in, b_in, c_in, d_in 
	        real(8), dimension(1,361) :: integral_1D 
            end subroutine blacksky_al
        end interface
   
        ! compute numerator
        N = chan_pos_end - chan_pos_start 

        ! for dhr integration
        do i = chan_pos_start, chan_pos_end
             call blacksky_al(i, sza, chl_con, wspd, wspd_dir, dhr)
          numerator(i) = stand_solar(i) * dhr
        enddo
        
        ! integral : numerator
        integral_n = 2 * h / 3.0D0 * 0.5 * (numerator(chan_pos_start) + numerator(chan_pos_end))
        do j = 1, N-1, 2
          integral_n = integral_n + 2.0D0 * h / 3.0D0 * (2.0D0 * numerator(chan_pos_start + j * 1))
        enddo
        do j = 2, N-2, 2
          integral_n = integral_n + 2.0D0 * h / 3.0D0 * numerator(chan_pos_start + j * 1)
        enddo
        final_numerator = integral_n
        ! integral : denominator
        integral_d = 2 * h / 3.0D0 * 0.5 * (stand_solar(chan_pos_start) + stand_solar(chan_pos_end))
        do j = 1, N-1, 2
          integral_d = integral_d + 2.0D0 * h / 3.0D0 * (2.0D0 * stand_solar(chan_pos_start + j * 1))
        enddo
        do j = 2, N-2, 2
          integral_d = integral_d + 2.0D0 * h / 3.0D0 * stand_solar(chan_pos_start + j * 1)
        enddo
        final_denominator = integral_d
        
        ! broadband albedo
        bd_dhr = final_numerator / final_denominator
        !bd_dhr_semiss = 1.0 - bd_dhr
        bd_dhr_semiss = bd_dhr

        ! for bhr integration use diffusivity angle (two-scheme)
        !diffuse_sza = abs(acos(1.0 / sqrt(3.0)) * 180.0 / pi)
        diffuse_sza = 53.0
        do i = chan_pos_start, chan_pos_end
          call blacksky_al(i, diffuse_sza, chl_con, wspd, wspd_dir, bhr)
          numerator_b(i) = stand_solar(i) * bhr
        enddo

        ! integral : numerator
        integral_nb = 2 * h / 3.0D0 * 0.5 * (numerator_b(chan_pos_start) + numerator_b(chan_pos_end))
        do j = 1, N-1, 2
          integral_nb = integral_nb + 2.0D0 * h / 3.0D0 * (2.0D0 * numerator_b(chan_pos_start + j * 1))
        enddo
        do j = 2, N-2, 2
          integral_nb = integral_nb + 2.0D0 * h / 3.0D0 * numerator_b(chan_pos_start + j * 1)
        enddo
        final_numerator_b = integral_nb
        ! integral : denominator
        integral_db = 2 * h / 3.0D0 * 0.5 * (stand_solar(chan_pos_start) + stand_solar(chan_pos_end))
        do j = 1, N-1, 2
          integral_db = integral_d + 2.0D0 * h / 3.0D0 * (2.0D0 * stand_solar(chan_pos_start + j * 1))
        enddo
        do j = 2, N-2, 2
          integral_db = integral_d + 2.0D0 * h / 3.0D0 * stand_solar(chan_pos_start + j * 1)
        enddo
        final_denominator_b = integral_db
        
        ! broadband albedo
        bd_bhr = final_numerator_b / final_denominator_b
        !bd_bhr_semiss = 1.0 - bd_bhr
        bd_bhr_semiss = bd_bhr
       
    end subroutine spctral_em

end module rrtmg_sw_ssal


!================================== subroutines --- abledo conponents ====================================
!
!
!----------------------------------- black sky albedo (rough sea surface) -------------------------------------
!
!-------------------------------------- whitecape reflectance ----------------------------------------
!
! References: Dierssen, H. M. (2019). Hyperspectral measurements, parameterizations, and atmospheric
!                     correction of whitecaps and foam from visible to shortwave infrared for ocean 
!                     color remote sensing. Earth Sci, 7, 14.
!
!------------------------------------ water-leaving reflectance --------------------------------------
!
! Reference: Morel, A., & Gentili, B. (1991). Diffuse reflectance of oceanic waters: its dependence 
!                 on Sun angle as influenced by the molecular scattering contribution. Applied optics,
!                 30(30), 4427-4438.
!            Morel, A., & Gentili, B. (1993). Diffuse reflectance of oceanic waters. II. Bidirectional
!                 aspects. Applied Optics, 32(33), 6864-6879.
!            Morel, A., & Gentili, B. (1996). Diffuse reflectance of oceanic waters. III. Implication
!                 of bidirectionality for the remote-sensing problem. Applied Optics, 35(24), 4850-4862.
!            Morel, A., & Maritorena, S. (2001). Bio‐optical properties of oceanic waters: A reappraisal. 
!                 Journal of Geophysical Research: Oceans, 106(C4), 7163-7180.
!            Bricaud, A., Morel, A., Babin, M., Allali, K., & Claustre, H. (1998). Variations of light
!                 absorption by suspended particles with chlorophyll a concentration in oceanic (case 1)
!                 waters: Analysis and implications for bio‐optical models. Journal of Geophysical 
!                 Research: Oceans, 103(C13), 31033-31044.
!            Lee, Z., Hu, C., Shang, S., Du, K., Lewis, M., Arnone, R., & Brewin, R. (2013). Penetration 
!                 of UV‐visible solar radiation in the global oceans: Insights from ocean color remote 
!                 sensing. Journal of Geophysical Research: Oceans, 118(9), 4241-4255.
!            Lee, Z., Du, K., Arnone, R., Liew, S., & Penta, B. (2005). Penetration of solar radiation in
!                 the upper ocean: A numerical model for oceanic and coastal waters. Journal of Geophysical
!                 Research: Oceans, 110(C9).
!            Brewin, R. J., Dall’Olmo, G., Sathyendranath, S., & Hardman-Mountford, N. J. (2012). Particle
!                 backscattering as a function of chlorophyll and phytoplankton size structure in the open
!                 ocean. Optics express, 20(16), 17632-17652.
!            Bellacicco, M., Cornec, M., Organelli, E., Brewin, R. J. W., Neukermans, G., Volpe, G. &. (2019).
!                 Global variability of optical backscattering by non‐algal particles from a biogeochemical‐
!                 argo data set. Geophysical Research Letters, 46(16), 9767-9776.
!            Vasilkov, A. P., Herman, J. R., Ahmad, Z., Kahru, M., & Mitchell, B. G. (2005). Assessment of 
!                 the ultraviolet radiation field in ocean waters from space-based measurements and full 
!                 radiative-transfer calculations. Applied Optics, 44(14), 2863-2869.
!            Varunan, T., & Shanmugam, P. (2015). A model for estimating size-fractioned phytoplankton 
!                 absorption coefficients in coastal and oceanic waters from satellite data. Remote Sensing 
!                 of Environment, 158, 235-254.
!            Brewin, R. J., Sathyendranath, S., Jackson, T., Barlow, R., Brotas, V., Airs, R., & Lamont, T. (2015).
!                 Influence of light in the mixed-layer on the parameters of a three-component model of phytoplankton
!                 size class. Remote Sensing of Environment, 168, 437-450.
!            Iuculano, F., Álvarez-Salgado, X. A., Otero, J., Catalá, T., Sobrino, C., Duarte, C., & Agustí,
!                 S. (2019). Patterns and drivers of UV absorbing chromophoric dissolved organic matter in the
!                 euphotic layer of the open ocean. Frontiers in Marine Science, 6, 320.
!            Wei, J., Lee, Z., Ondrusek, M., Mannino, A., Tzortziou, M., & Armstrong, R. (2016). Spectral 
!                 slopes of the absorption coefficient of colored dissolved and detrital material inverted from 
!                 UV‐visible remote sensing reflectance. Journal of Geophysical Research: Oceans, 121(3), 1953-1969.
!            Bricaud, A., Babin, M., Claustre, H., Ras, J., & Tièche, F. (2010). Light absorption properties and 
!                 absorption budget of Southeast Pacific waters. Journal of Geophysical Research: Oceans, 115(C8).
!            Bricaud, A., Morel, A., Babin, M., Allali, K., & Claustre, H. (1998). Variations of light absorption by
!                 suspended particles with chlorophyll a concentration in oceanic (case 1) waters: Analysis and
!                 implications for bio‐optical models. Journal of Geophysical Research: Oceans, 103(C13), 31033-31044.
!            Gordon, H. R. (2005). Normalized water-leaving radiance: revisiting the influence of surface roughness.
!                 Applied Optics, 44(2), 241-248.
!            Twardowski, M.S.; Boss, E.; Macdonald, J.B.; Pegau, W.S.; Barnard, A.H.; Zaneveld, J.R.V. A model for
!                 estimating bulk refractive index from the optical backscattering ratio and the implications for 
!                 understanding particle composition in Case I and Case II waters. J. Geophys. Res. 2001, 106, 
!                 14129–14142.
!------------------------------------ sun glint reflectance --------------------------------------
! References: Cox, C., & Munk, W. (1954). Measurement of the roughness of the sea surface from photographs
!                 of the sun’s glitter. Josa, 44(11), 838-850.
!             Zhai, P. W., Hu, Y., Chowdhary, J., Trepte, C. R., Lucker, P. L., & Josset, D. B. (2010). A
!                 vector radiative transfer model for coupled atmosphere and ocean systems with a rough 
!                 interface. Journal of Quantitative Spectroscopy and Radiative Transfer, 111(7-8), 1025-1040.
!             Harmel, T., & Chami, M. (2013). Estimation of the sunglint radiance field from optical satellite
!                 imagery over open ocean: Multidirectional approach and polarization aspects. Journal of 
!                 Geophysical Research: Oceans, 118(1), 76-90.
!             Schaepman-Strub, G., Schaepman, M. E., Painter, T. H., Dangel, S., & Martonchik, J. V. (2006). 
!                 Reflectance quantities in optical remote sensing—Definitions and case studies. Remote 
!                 sensing of environment, 103(1), 27-42.
!             Albert, M. F., Anguelova, M. D., Manders, A. M., Schaap, M., & De Leeuw, G. (2016). Parameterization
!                 of oceanic whitecap fraction based on satellite observations. Atmospheric Chemistry and Physics.
!
! This is a scalar calculation process.
!
    subroutine blacksky_al(loc, sza, chl_con, wspd, wspd_dir, dhr)
        use rrsw_ocaux
        implicit none

        ! declare variables in & out
        integer, intent(in) :: loc             ! position of wavelength
        real(8), intent(in) :: sza             ! solar zenith angle in degree
        real(8), intent(in) :: chl_con         ! chlorophyll concentration (mg/m-3)
        real(8), intent(in) :: wspd            ! wind speed (m/s)
        real(8), intent(in) :: wspd_dir        ! wind direction in degree [0.0 ~ 360.0)
        real(8), intent(out) :: dhr            ! blacksky albedo

        ! declare varibles for 'whitecaps'
        real(8) :: temp, wc_refl               ! whitecap reflectance
        real(8), parameter :: a0=0.47, a1 = - 1.62, a2 = - 8.66, a3 = 31.81       ! constents

        ! declare varibles for water leaving, 
        real(8), parameter :: p_m = 0.043      ! constents
        ! parameters for calculating absorption and backscattering
        real(8) :: ab_cdom440, ab_cdom, ab_nap440, ab_nap, ab_phy, C_pn, C_p, C_n, C_m, abs_tots
        real(8) :: lamda0, Cm12, S12, bp12wl0, bp3wl0, bpkwl0, gama12, gama3, gamak, bbp, bb_tots
        ! variables for subsurface remote sensing reflectance
        real(8), parameter :: pi = acos(- 1.0)
        real(8) :: vza(91) = (/(iv,iv=0,90,1)/)
        real(8) :: aza(361) = (/(iv,iv=0,360,1)/)
        real(8) :: ui, sin_sza_w, Kd, ita_bb, bb_a, V, e1, e2, e3, e4, e5, e6, e7, e8, &
                   e9, e10, e11, e12, e13, e14, e15, e16, e17, e18, H, P3u, sza_w, &
                   Mdp, mds1, mds2, mds3, mds4, mds5, mds6, mds7, mds8, Mds, &
                   audp, audp_infi, fa1, fa2, fa3, fa4, fa5, bb_tots_ratio
        real(8), dimension(91) :: uv
        real(8), dimension(361) :: uaz
        real(8) :: uscat_wl, angscat_wl, pf_bp, pf_bw, faup, vsf, rrs, wl_refl
        real(8) :: refid_rwa, refid_raw, sintheta_wp, Rv, Rp, Rvp, Rwlr_tem

        ! declare variables for 'sun glint'
        real(8) :: geda, etax, A_ui, shadow_x, wspd_dir_m, geda_x, geda_y
        real(8) :: C21, C03, C40, C22, C04
        real(8), dimension(91) :: etay, A_uv, shadow_y, shadow_c
        real(8), dimension(91, 361) :: uscat, angscat, unorm, locin_ang
        real(8), dimension(91, 361) :: Zx, Zy, Zx_n, Zy_n, ksai_x, Ksai_y
        real(8), dimension(91, 361) :: clocin, slocin, Rvu, Rpu, Rvpu
        real(8), dimension(91, 361) :: GCs, PDF, denomi
        real(8), dimension(91, 361) :: sg_refl
        integer :: iuv, iuaz
       
        ! decalare for integration
        real(8) :: wc_fra
        real(8), dimension(91, 361):: BRDF, infunc
        real(8)       :: a, b, c, d, hx, hy, integral, integral_2D, dhr_temp
        integer                :: i, j, NX, NY, a_in, b_in, c_in, d_in 
        real(8), dimension(1,361) :: integral_1D

        !============================== whitecaps reflectance
        if (wl(loc) >= 380.0 .and. wl(loc) <= 2400.0) then
            temp = log10(abs_sw(loc))
            wc_refl = 0.01 * (a0 * temp ** (3.0) + a1 * temp **(2.0) + a2 * temp + a3)
        else
            wc_refl = 0.0
        endif

        !=============================== absorption
        ! absorption coefficient of CDOM 
        ! ab_cdom440 from Morel et al., (2001)
        ! 200 - 275nm: just reseasonalble guess
        ! 275 - 300nm: average value in figure7-(d) from Iuculano et al., (2019)
        ! 300 - 400nm: average value from Wei et al., (2016)
        ! > 400nm: from Bricaud et al., (1998)
        !

         ab_cdom440 = 0.2 * (0.00522 + 0.06 * chl_con ** (0.65))
         if (wl(loc) < 275.0) then
             ab_cdom = ab_cdom440 * exp(- 0.028 * (wl(loc) - 440))   ! S_cdom = 0.028
         elseif (wl(loc) >= 275.0 .and. wl(loc) < 300.0) then
             ab_cdom = ab_cdom440 * exp(- 0.025 * (wl(loc) - 440))   ! S_cdom = 0.025
         elseif (wl(loc) >= 300.0 .and. wl(loc) <= 700.0) then
             ab_cdom = ab_cdom440 * exp(- 0.014 * (wl(loc) - 440))   ! S_cdom = 0.014
         else
             ab_cdom = 0.0
         endif
        
         ! absorption coefficient of non-algae
         ! ab_nap440 from Bricaud et al., (2010)
         ! 200 - 275nm: just reseaonable guess
         ! 275 - 300nm: average value page 5 (close to Eq.(2)) in Wei et al., (2016)
         ! 300 - 400nm: ab_nap = 0.0 since the 'ab_phy' has already defined
         ! MID-range UV particle absorption (phytoplankton plus detritus) 
         ! 400 - 700nm: from www.oceanopticsbook.info
        !
         ab_nap440 = 0.0067 * chl_con ** 0.675
         if (wl(loc) < 275.0) then
             ab_nap = ab_nap440 * exp(- 0.016 * (wl(loc) - 440))      ! S_nap = 0.016
         elseif (wl(loc) >= 275.0 .and. wl(loc) < 300.0) then
             ab_nap = ab_nap440 * exp(- 0.014 * (wl(loc) - 440))      ! S_nap = 0.014
         elseif (wl(loc) >= 300.0 .and. wl(loc) < 400.0) then
             ab_nap = 0.0 
         elseif (wl(loc) >= 400.0 .and. wl(loc) <= 700.0) then
             ab_nap = ab_nap440 * exp(- 0.011 * (wl(loc) - 440))      ! S_nap = 0.011
         else
             ab_nap = 0.0
         endif
        
        ! absorption coefficient of phytoplankton; 
        ! 200 - 300nm: no phytoplankton absorption;
        ! 300 - 400nm: use power - law: a*P(wl) = A(wl) * chl_con ** (- B(wl)); from Vasilkov et al., (2005)
        ! 400 - 700nm: use size-fractioned phytoplankton absorption; from Vanrunan et al., (2015) & Brewin et al., (2015)
        ! E.q.(18)& (19) in Vanrunan et al.,(2015)
        ! E.q.(18)& (19) in Brewin et al. (2015)
        ! C_pn = C^m_pn * (1.0 - exp(- D_pn / C^m_pn) * Chl_con)
        ! C_p = C^m_p * (1.0 - exp(- D_p / C^m_p) * Chl_con)
        ! C^m_pn = 0.77;C^m_p = 0.13;D_pn = 0.94;D_p =0.80
        !
        C_pn = 0.77 * (1.0 - exp(- (0.94 / 0.77) * chl_con))
        C_p = 0.13 * (1.0 - exp(- (0.80 / 0.13) * chl_con))
        C_n = C_pn - C_p
        C_m = chl_con - C_pn

          if (chl_con == 0.0) then
             ab_phy = 0.0
         else
             if (wl(loc) < 300.0) then
               ab_phy = 0.0
             elseif (wl(loc) >= 300.0 .and. wl(loc) < 400.0) then
               ab_phy = specific_A(loc) * chl_con ** (specific_B(loc))
             elseif (wl(loc) >= 400.0 .and. wl(loc) <= 700.0) then
               ab_phy = specific_ap(loc) * C_p + specific_an(loc) * C_n + specific_am(loc) * C_m
             else
               ab_phy = 0.0
             endif
         endif
        
        ! total absorption coefficient
        abs_tots = abs_sw(loc) + ab_cdom + ab_nap + ab_phy        

        !=============================== scattering
        !
        ! particle backscattering model from  Brewin et al., (2012)
        !
        ! characters             parameters                         units
        !
        ! lamda0                = lamda0                              nm
        ! Cm12                  = C^m_12                           mg m^(-3)
        ! S12                   = S12 * C^m _12 / C^m _12         m^3 mg^(-1)
        ! bp12wl0               = bp^* _12(wl0)                mg^2 (mg C)^(-1)
        ! bp3wl0                = bp^* _3(wl0)                 mg^1 (mg C)^(-1)
        ! bpkwl0                = bp^* _k(wl0)                      m^(-1)
        ! gama12                = r12                           dimensionless
        ! gama3                 = r3                            dimensionless
        ! gamak                 = rk                            dimensionless

        lamda0 = 470 
        Cm12 = 0.780 
        S12 = 0.893 / Cm12
        bp12wl0 = 0.0046
        bp3wl0 = 0.0005
        bpkwl0 = 0.00068
        gama12 = 0.7
        gama3 = - 0.2
        gamak = 1.9
        ! particle backscattering coefficient
        bbp = bp12wl0 * (wl(loc) / lamda0) ** (- gama12) * Cm12 * (1.0 - exp(- S12 * chl_con)) + &
                bp3wl0 * (wl(loc) / lamda0) ** (- gama3) * (chl_con - Cm12 * (1.0 - exp(- S12 * chl_con))) + &
                bpkwl0 * (wl(loc) / lamda0) ** (- gamak)
        
        ! total backscattering coefficient
        bb_tots = bbp + 0.5 * sca_sw(loc)

        !============================== define incident, viewing, azimuth angles
        ! 'vza': view zenith angle; 'aza': azimuth angle
        ! 'ui': consine of sza; 'uv': cosine of view angle
        if (sza == 0.0) then
            ui = 0.999999999999999
        elseif (sza == 90.0) then
            ui = abs(cos(pi * 90.0 / 180.0))
        else
            ui = cos(pi * sza / 180.0)
        endif        
        uv = cos(pi * vza / 180.0)
        uv(1) = 0.999999999999999
        uv(91) = abs(cos(pi * 90.0 / 180.0))
        uaz = cos(pi * aza / 180.0)
        uaz(91) = 0.0; uaz(271) = 0.0

        !============================== scatter angle & surface normal line & zenith angle of normal line
        !  compute scatter angle and the angle of normal line
        ! 'uscat_wl':consine of scatter angle in water; 
        ! 'uscat':consine of scatter angle;
        ! 'unorm':cosine of zenith angle of normal line

        sin_sza_w = (1.000277 / refid(loc)) * sqrt(1.0 - ui ** 2.0)
        sza_w = asin(sin_sza_w) * 180.0 / pi 
        uscat_wl = cos(pi * sza_w / 180.0) * uv(1)           
        ! scatter angel in degree
        if (uscat_wl > 1.0) then
           uscat_wl = 1.0
        else
           uscat_wl = uscat_wl
        endif
        angscat_wl = acos(uscat_wl) * 180.0 / pi
        angscat_wl = real(angscat_wl)

        do iuv = 1, 91
          do iuaz = 1,361
            uscat(iuv, iuaz) = ui * uv(iuv) + sqrt(1.0 - ui ** 2.0) * sqrt(1.0 - uv(iuv) ** 2.0) * uaz(iuaz)
            unorm(iuv, iuaz) = (uv(iuv) + ui) / sqrt(2.0 + 2.0 * uscat(iuv, iuaz))
          enddo
        enddo
        ! scatter angel in degree
        where (uscat > 1.0)
            uscat = 1.0
        elsewhere
            uscat = uscat
        endwhere
        angscat = acos(uscat) * 180.0 / pi
        angscat = real(angscat)
        
        ! local incident angle of light 
        locin_ang = 90.0 - (180.0 - angscat) / 2.0


        !============================== calculate ZTT model: calculate subsurface remote sensing reflectance
        ! diffuse attenuation coefficients for downward spectrum;
        ! 'Kd'(upper letter 'K'); Lee et al., 2013 for UV-VIS; Lee et al., 2005 for infrared
        if (wl(loc) <= 780.0) then
            Kd = (1.0 + 0.005 * sza) * abs_tots + (1.0 - 0.265 * (0.5 * sca_sw(loc) / bb_tots)) * 4.259 * &
                 (1.0 - 0.52 * exp(- 10.8 * abs_tots)) * bb_tots
        else
            Kd = (0.560 + 2.304 / ((0.001 + 0.0) ** 0.65)) * (1.0 + 0.002 * sza)
        endif         
        
        ! the fraction of b_b attributable to molecular scattering
        ita_bb = 0.5 * sca_sw(loc) / bb_tots     
        bb_a = bb_tots / abs_tots
        ! the average consine of downwelling radiance;Twardski et al., 2018 He et al.,2017
        audp = 0.4779 * (1.0 - 0.0641 * cos(pi * sza_w / 180.0) + 0.2660 * (cos(pi * sza_w /180.0)) ** 2.0) * &
               (1.0 + 0.7164 * abs_tots / Kd)
 
        ! Average Cosine of the Asymptotic Light Field; Twardowski & Albert., 2017,2018
        audp_infi = 0.0679 - 0.020 * log(bb_a) - 0.0808 * (log(bb_a)) ** 2.0 + 0.3859 * ita_bb - &
                    0.0837 * ita_bb * log(bb_a) - 0.490 * ita_bb * (log(bb_a)) ** 2.0 + 0.573 * ita_bb ** 2.0 - &
                    0.0743 * ita_bb ** 2.0 + 0.233 * ita_bb ** 2.0 * (log(bb_a)) ** 2.0
        
        ! Fai & Phase functions 
        fa1 = - 3.79435531537314 * 10 ** (- 7.0)
        fa2 = 2.42117623125973 * 10 ** (- 4.0)
        fa3 = - 5.76056692150838 * 10 ** (- 2.0)
        fa4 = 6.04944577004764
        fa5 = - 236.166389774491
        
        if (angscat_wl >= 90.0 .and. angscat_wl <= 180.0) then
            angscat_wl = angscat_wl
        else
            angscat_wl = 0.0
        endif
        ! Average ST PF
        pf_bp = 4.722e-09 * angscat_wl ** 4.0 - 2.904e-06 * angscat_wl ** 3.0 + &
                0.0006774 * angscat_wl ** 2.0 - 0.07076 * angscat_wl + 2.92
        
        if (angscat_wl == 0.0) then
           pf_bw = 0.0
           faup = 0.0
        else
           pf_bw = 0.06225 * (1.0 + 0.835 * cos(pi * angscat_wl / 180.0) ** (2.0))
           faup = fa1 * angscat_wl ** 4.0 + fa2 * angscat_wl ** 3.0 + &
                  fa3 * angscat_wl ** 2.0 + fa4 * angscat_wl + fa5 + 1.0 
        endif        

        vsf = pf_bp * bbp + pf_bw * 0.5 *sca_sw(loc)
        ! backscattering ratio bb_tots_ratio; Average Petzold Phase function, 
        ! from Morel et al, 2001
        bb_tots_ratio = bb_tots / (bbp / (0.002 + (0.01 * (0.5 - 0.25 * log10(chl_con)))) + sca_sw(loc))
        
        ! subsurface remote sensing reflectance
        rrs = (1.0 / audp) * (vsf / bb_tots) / &
              ((abs_tots / bb_tots) * (1.0 + uv(1) * faup * (audp_infi) ** (- 1.0)) + &
              1.05 * (1.0 - bb_tots_ratio ** (- 1.0)) + bb_tots_ratio ** (- 1.0))

        ! Fresnel reflectance below surface
        ! air refractive: 1.000277; relative refractive: water to air ; relative refractive: air to water
        refid_rwa = refid(loc) / 1.000277
        refid_raw = 1 / refid_rwa
        ! refractive angle 
        sintheta_wp = sin(pi * sza / 180.0) / refid_rwa
        ! full relection or not
        if (refid_raw * sintheta_wp >= 1.0) then
            Rv = 1.0 ! vertical
            Rp = 1.0 ! parallel
        else 
            Rv = ((1.0 - sintheta_wp ** 2.0) ** 0.5 - (refid_raw ** 2.0 - sintheta_wp ** 2.0) ** 0.5) / &
                ((1.0 - sintheta_wp ** 2.0) ** 0.5 + (refid_raw ** 2.0 - sintheta_wp ** 2.0) ** 0.5)
            Rp = (refid_raw * (1.0 - sintheta_wp ** 2.0) ** 0.5 - (refid_raw ** 2.0 - sintheta_wp ** 2.0) ** 0.5) / &
                (refid_raw * (1.0 - sintheta_wp ** 2.0) ** 0.5 + (refid_raw ** 2.0 - sintheta_wp ** 2.0) ** 0.5) 
        endif
        Rvp = 0.5 * (Rv ** 2.0 + Rp ** 2.0)
        
        !============================== calculate water leaving reflectance     
        Rwlr_tem = ((1.0 - p_m) * (1.0 - Rvp)) / ((refid_rwa ** 2.0) * 0.985)
        wl_refl = Rwlr_tem * rrs / pi

        !============================== calculate sunglint reflectance
        ! shadow factor
        geda = (0.003 + 0.00512 * wspd) ** (0.5)
        geda_x = (0.003 + 0.00192 * wspd) ** 0.5
        geda_y = (0.000 + 0.00316 * wspd) ** 0.5
        ! shadow function
        etax = ui / (geda_x * sqrt (1.0 - ui ** 2.0))
        do iuv = 1, 91
          etay(iuv) = uv(iuv) /(geda_y * sqrt (1.0 - uv(iuv) ** 2.0))
        enddo
        A_ui = 0.5 * ((1.0 / sqrt(pi)) * (1.0 / etax) * exp(- etax ** 2.0) - erfc(etax))
        A_uv = 0.5 * ((1.0 / sqrt(pi)) * (1.0 / etay) * exp(- etay ** 2.0) - erfc(etay))
        shadow_x = 1.0 / (1.0 + A_ui)
        shadow_y = 1.0 / (1.0 + A_uv)

        ! rough & slope surface
        ! find surface slope components;
        do iuv = 1, 91
          do iuaz = 1, 361
            Zx(iuv, iuaz) = - sin(vza(iuv) * pi / 180.0) * sin(aza(iuaz) * pi / 180.0) / (ui + uv(iuv))
            Zy(iuv, iuaz) = (sin(sza * pi / 180.0) + sin(vza(iuv) * pi / 180.0) * uaz(iuaz)) / (ui + uv(iuv))
          enddo
        enddo
        ! wind direction: rotated clockwise from North
        wspd_dir_m = wspd_dir * pi / 180.0
        ! find slope components in a new coordinate with wind direction
        Zx_n = cos(wspd_dir_m) * Zx + sin(wspd_dir_m) * Zy
        Zy_n = - sin(wspd_dir_m) * Zx + cos(wspd_dir_m) * Zy
        ksai_x = Zx_n / geda_x
        ksai_y = Zy_n / geda_y

        ! Fresnel reflectance below surface
        ! surface fresnel reflectance calculation
        ! air refractive: 1.000277; 'refid_rwa': relative refractive: water to air ;
        !refid_rwa = refid(loc) / 1.000277
        
        where (locin_ang == 0.0)
            clocin = 0.999999999999999
        elsewhere (locin_ang == 90.0)
            clocin = abs(cos(90.0 * pi / 180.0))
        elsewhere
            clocin = cos(locin_ang * pi / 180.0)
        endwhere
        slocin = sin(locin_ang * pi / 180.0)
        Rvu = (clocin - (refid_rwa ** 2.0 - slocin ** 2.0) ** 0.5) / &
          (clocin + (refid_rwa ** 2.0 - slocin ** 2.0) ** 0.5)
        Rpu = ((refid_rwa ** 2.0) *clocin - (refid_rwa ** 2.0 - slocin ** 2.0) ** 0.5) / &
          ((refid_rwa ** 2.0) * clocin + (refid_rwa ** 2.0 - slocin ** 2.0) ** 0.5)
        Rvpu = 0.5 * (Rvu ** 2.0 + Rpu ** 2.0)

        !============================== probability distribution functione
        ! probability distribution function; PDF, assuming isotropic cox-munk model;
        ! Gram-Charlier series (Cox & Muck., 1954)
        C21 = 0.01 - 0.0086 * wspd
        C03 = 0.04 - 0.033 * wspd
        C40 = 0.04
        C22 = 0.12
        C04 = 0.23
        GCs = 1.0 - 0.5 * C21 * (ksai_x ** 2.0 - 1.0) - (1.0 / 6.0) * C03 * (ksai_y ** 3.0 - ksai_y) + &
          (1.0 / 24.0) * C40 * (ksai_x ** 4.0 - 6.0 * ksai_x ** 2.0 - 3.0) + &
          (1.0 / 4.0) * C22 * (ksai_x ** 2.0 - 1.0) * (ksai_y ** 2.0 - 1.0) + &
          (1.0 / 24.0) * C04 * (ksai_y ** 4.0 - 6.0 * ksai_y ** 2.0 - 3.0)
        PDF = (1.0 / (2.0 * pi * geda_x * geda_y)) * exp(- 0.5 * (ksai_x ** 2.0 + ksai_y ** 2.0)) * GCs
        ! total rough sea surface reflectance
        !'denomi': denominator of total rough sea surface reflectance
        do iuv = 1, 91
          denomi(iuv,:) = ui * uv(iuv) * unorm(iuv,:)
        enddo
        ! shadow factor correction
        shadow_c = shadow_x * shadow_y

        ! rough sea surface reflectance
        do iuv = 1, 91
            do iuaz = 1, 361
              sg_refl(iuv, iuaz) = (pi / 4.0) * shadow_c(iuv) * PDF(iuv, iuaz) * &
                  Rvpu(iuv, iuaz) * (1.0 / denomi(iuv, iuaz))
            enddo
        enddo

        !============================== whitecaps fraction
        ! whitecaps fraction (wind speed dependent)
        wc_fra = (10.77 * 10 ** (- 5.0)) * (wspd + 1.789) ** 2.0

        !============================== BRDF reflectance                
        BRDF = wc_fra * wc_refl + (1.0 - wc_fra) * (wl_refl + sg_refl)

        !============================== initialization: integral region & steps
        !
        !----------------------------- description ---------------------------
        ! this subrouine use Composite Simpson's rule to do double integral a
        ! function with rectangular region (integral regions are fixed)
        !
        ! variables and parameters used in the following:
        ! a                             = Lower bound of integral on 'vza' axis
        ! b                             = Upper bound of integral on 'vza' axis
        ! c                             = Lower bound of integral on 'aza' axis
        ! d                             = Upper bound of integral on 'aza' axis
        ! a_in, b_in, c_in, d_in        = index of 'infunc' // 'integral_1D'
        ! NX                            = Number of divisions on 'vza' axis
        ! NY                            = Number of divisions on 'aza' axis
        ! hx                            = Step size on 'vza' axis
        ! hy                            = Step size on 'aza' axis
        ! integral_1D                   = Result of integral process of 'vza'
        ! integral_2D                   = Result of integral process of both 'vza' and 'aza'
        ! i,j                           = Integer for counting
        !
        ! initialize limit of integration, rectangular integral region; view angle: [0 90]; azimuth angle: [0 360]
        ! intergral function 'infunc'
        do iuv = 1, 91
          infunc(iuv, :) = sin(pi * vza(iuv) / 180.0) * uv(iuv) * BRDF(iuv, :)
        enddo
        
        a = 0.0D0
        b = pi / 2.0D0                               
        c = 0.0D0
        d = pi * 2.0D0
        NX = 45
        NY = 180
        hx = (b - a) / (2 * NX)
        hy = (d - c) / (2 * NY)

        ! double integral procedure
        ! integral 'vza' 
        do j =1, 361
          a_in = nint(180.0 / pi * a) + 1
          b_in = nint(180.0 / pi * b) + 1
          integral = 2 * hx / 3.0D0 * 0.5D0 * (infunc(a_in, j) + infunc(b_in, j))
          do i = 1, 2*NX-1, 2
            integral = integral + 2.0D0 * hx / 3.0D0 * (2.0D0 * infunc(a_in + i * 1, j))
          enddo
          do i = 2, 2*NX-2, 2
            integral = integral + 2.0D0 * hx / 3.0D0 * infunc(a_in + i * 1, j)
          enddo
            integral_1D(1,j) = integral
        enddo

        ! integral 'aza'
        c_in = nint(180.0 / pi * c) + 1
        d_in = nint(180.0 / pi * d) + 1

        integral = 2 * hy / 3.0D0 * 0.5D0 * (integral_1D(1, c_in) + integral_1D(1, d_in))
        do i = 1, 2*NY-1, 2
         integral = integral + 2.0D0 * hy / 3.0D0 * (2.0D0 * integral_1D(1, c_in + i * 1))
        enddo
        do i = 2, 2*NY-2, 2
          integral = integral + 2.0D0 * hy / 3.0D0 * integral_1D(1, c_in + i * 1)
        enddo
        integral_2D = integral
       
        ! black sky sky albedo
        dhr_temp = (1.0 / pi) * integral_2D
        
        if (dhr_temp >= 1.0) then
            dhr = 1.0
        else
            dhr = dhr_temp
        endif
            
    end subroutine blacksky_al







