!==========================================================================================================!
! This module provides some anxiliary data for calculating sea surface albedo of ocean optics              !
!                                                                                                          !
! Parameters:                                                                                              !
! 'wl_default': wavelength                                                                                 ! 
! 'refid_defaul': refractive index of sea water                                                            !
! 'abs_sw_default': absorption coefficients of pure sea water; unit: m^(-1)                                !
! 'sca_sw_default': scattering coefficients of pure sea water; unit: m^(-1)                                !
! 'stand_solar_default': standard irradiance solar spectra; unit: W/m^2/nm                                 !
!                                                                                                          !
! specific absorption coefficients of phytoplankton: applicable: less than 400nm; unit: m ** 2 / mg        !
! power-law: a*p(wl) = A(wl) * chl_con ** (B(wl))                                                          !
! 'specific_A_default': A(wl); 'specific_B_default': B(wl)                                                 !
!                                                                                                          !
! specific absorption coefficients of phytoplankton: applicable: 400 - 700nm; unit: m ** 2 / mg            !
! size-fraction phytoplankton absoption coefficients                                                       !
! three size: pico- (2nm<), nano- (2-20nm), micro-phytoplankton (>20nm)                                    !
! 'specific_ap_default'; 'specific_an_default'; 'specific_am_fault';                                       !
!                                                                                                          !
! Reference: Röttgers, R., Doerffer, R., McKee, D., & Schönfeld, W. (2010). Pure water spectral absorption,!
!                scattering, and real part of refractive index model Algorithm Technical Basis Document.   !
!            Mason, J. D., Cone, M. T., & Fry, E. S. (2016). Ultraviolet (250–550 nm) absorption spectrum  !
!                of pure water. Applied optics, 55(25), 7163-7172.                                         !
!            ASTM, E. (2000). 490-00. Standard Extraterrestrial Spectrum Reference.                        !
!            Mobley, C., Boss, E., & Roesler, C. (2010). Ocean optics web book,phytoplankton specific      !
!                absortion coefficient.                                                                    !
!            2000 ASTM Standard Extraterrestrial Spectrum Reference E-490-00.                              !
!            https://www.nrel.gov/grid/solar-resource/spectra-astm-e490.html                               !
!==========================================================================================================!
!                                                                                                          !
! Date: Sep.,2020                                                                                          !
! Author: Jian Wei.       Email: anser@tamu.edu      or     anserwei@gmail.com                             !
! Atmospheric, Oceanic Optics and Remote Sensing Lab, Texas A&M University                                 !
!                                                                                                          !
!==========================================================================================================!

module rrsw_ocaux
    implicit none

! wl: wavelength in nm     
     integer :: iv 
     integer, parameter :: wl(913) = (/(iv,iv=200,3848,4)/)
     
     
!     integer, dimension(925) :: wl

! 'chan_pos': RRTMG channel position: each coloumn indicates one band; 1st row & 2nd row: wavelength range
    !
    !Band #   Wavenumber (cm-1)   (nm)           1050 - 96 mb      96 - 0.01 mb       chan_pos (see rrsw_ocaux.f90)
    !16        2600-3250    (3846.1-3076.9)         H2O,CH4             CH4                chan_pos(2,13) - chan_pos(1,13)
    !17        3250-4000    (3076.9-2500.0)         H2O,CO2           H2O,CO2              chan_pos(2,12) - chan_pos(1,12)
    !18        4000-4650    (2500.0-2150.5)         H2O,CH4             CH4                chan_pos(2,11) - chan_pos(1,11)
    !18        4650-5150    (2150.5-1941.8)         H2O,CO2             CO2                chan_pos(2,10) - chan_pos(1,10)
    !20        5150-6150    (1941.8-1626.0)         H2O,CH4*          H2O,CH4*              chan_pos(2,9) - chan_pos(1,9)
    !21        6150-7700    (1626.0-1298.7)         H2O,CO2           H2O,CO2               chan_pos(2,8) - chan_pos(1,8)
    !22        7700-8050    (1298.7-1242.4)         H2O,O2              O2                  chan_pos(2,7) - chan_pos(1,7)
    !23        8050-12850   (1242.4- 778.2)           H2O             nothing               chan_pos(2,6) - chan_pos(1,6)
    !24       12850-16000   (778.2 - 625.0)        H2O,O2,O3*          O2,O3*               chan_pos(2,5) - chan_pos(1,5)
    !25       16000-22650   (625.0 - 441.5)         H2O,O3*             O3*                 chan_pos(2,4) - chan_pos(1,4)
    !26       22650-29000   (441.5 - 344.8)         nothing           nothing               chan_pos(2,3) - chan_pos(1,3)
    !20       29000-38000   (344.8 - 263.1)           O3                O3                  chan_pos(2,2) - chan_pos(1,2)
    !28       38000-50000   (263.1 - 200.0)          O3,O2            O3,O2                 chan_pos(2,1) - chan_pos(1,1)
    !29         820-2600   - no applicable -          H2O               CO2                   - - - not applicable - - - :
     integer, parameter, dimension(2,13) :: chan_pos = reshape((/1, 17, 17, 37, 37, 61, 61, 107, 107, 146, 146, 262, & 
             262, 276, 276, 358, 358, 436, 436, 489, 489, 576, 576, 720, 720, 913/), (/2, 13/))


! 'refid': refractive index of sea water

     real(8), parameter, dimension(913)::refid = (/1.4306796572, &
     1.4259415095,   1.4214567544,   1.4172161041,   1.4132102707, & 
     1.4094299665,   1.4058658862,   1.4025086554,   1.3993488908,   1.3963774086, & 
     1.3935852245,   1.3909631026,   1.3885007668,   1.3861880019,   1.3840185548, & 
     1.3819822853,   1.3800687392,   1.3782684278,   1.3765727265,   1.3749737851, & 
     1.3734644484,   1.3720381860,   1.3706890303,   1.3694115206,   1.3682006545, & 
     1.3670518436,   1.3659608745,   1.3649238737,   1.3639372762,   1.3629977975, & 
     1.3621024083,   1.3612483118,   1.3604329231,   1.3596538511,   1.3589088817, & 
     1.3581959626,   1.3575131900,   1.3568587962,   1.3562311382,   1.3556286880, & 
     1.3550500229,   1.3544938176,   1.3539588360,   1.3534439248,   1.3529480069, & 
     1.3524700754,   1.3520091887,   1.3515644653,   1.3511350796,   1.3507202576, & 
     1.3503192733,   1.3499314452,   1.3495561332,   1.3491927357,   1.3488406867, & 
     1.3484994536,   1.3481685349,   1.3478474581,   1.3475357773,   1.3472330723, & 
     1.3469389461,   1.3466530239,   1.3463749512,   1.3461043931,   1.3458410326, & 
     1.3455845694,   1.3453347195,   1.3450912134,   1.3448537956,   1.3446222238, & 
     1.3443962680,   1.3441757099,   1.3439603418,   1.3437499666,   1.3435443965, & 
     1.3433434531,   1.3431469665,   1.3429547748,   1.3427667239,   1.3425826667, & 
     1.3424024631,   1.3422259793,   1.3420530877,   1.3418836664,   1.3417175990, & 
     1.3415547740,   1.3413950853,   1.3412384308,   1.3410847134,   1.3409338396, & 
     1.3407857203,   1.3406402700,   1.3404974066,   1.3403570515,   1.3402191294, & 
     1.3400835679,   1.3399502977,   1.3398192520,   1.3396903669,   1.3395635808, & 
     1.3394388346,   1.3393160714,   1.3391952366,   1.3390762775,   1.3389591434, & 
     1.3388437857,   1.3387301573,   1.3386182130,   1.3385079092,   1.3383992038, & 
     1.3382920562,   1.3381864275,   1.3380822797,   1.3379795764,   1.3378782826, & 
     1.3377783641,   1.3376797882,   1.3375825232,   1.3374865383,   1.3373918038, & 
     1.3372982912,   1.3372059727,   1.3371148213,   1.3370248111,   1.3369359169, & 
     1.3368481143,   1.3367613797,   1.3366756902,   1.3365910236,   1.3365073583, & 
     1.3364246735,   1.3363429489,   1.3362621649,   1.3361823024,   1.3361033430, & 
     1.3360252685,   1.3359480617,   1.3358717054,   1.3357961833,   1.3357214793, & 
     1.3356475778,   1.3355744638,   1.3355021224,   1.3354305393,   1.3353597007, & 
     1.3352895929,   1.3352202027,   1.3351515173,   1.3350835242,   1.3350162111, & 
     1.3349495662,   1.3348915662,   1.3348345662,   1.3347795662,   1.3347245662, & 
     1.3346705662,   1.3346175662,   1.3345645662,   1.3345115662,   1.3344595662, & 
     1.3344075662,   1.3343555662,   1.3343045662,   1.3342545662,   1.3342045662, & 
     1.3341545662,   1.3341045662,   1.3340555662,   1.3340065662,   1.3339575662, & 
     1.3339095662,   1.3338615662,   1.3338135662,   1.3337665662,   1.3337195662, & 
     1.3336725662,   1.3336255662,   1.3335795662,   1.3335325662,   1.3334865662, & 
     1.3334415662,   1.3333955662,   1.3333505662,   1.3333055662,   1.3332605662, & 
     1.3332155662,   1.3331715662,   1.3331275662,   1.3330825662,   1.3330385662, & 
     1.3329955662,   1.3329515662,   1.3329085662,   1.3328645662,   1.3328215662, & 
     1.3327785662,   1.3327355662,   1.3326925662,   1.3326495662,   1.3326065662, & 
     1.3325635662,   1.3325205662,   1.3324785662,   1.3324355662,   1.3323935662, & 
     1.3323515662,   1.3323095662,   1.3322665662,   1.3322245662,   1.3321825662, & 
     1.3321405662,   1.3320985662,   1.3320565662,   1.3320145662,   1.3319725662, & 
     1.3319305662,   1.3318875662,   1.3318455662,   1.3318035662,   1.3317615662, & 
     1.3317195662,   1.3316775662,   1.3316355662,   1.3315935662,   1.3315515662, & 
     1.3315095662,   1.3314675662,   1.3314255662,   1.3313835662,   1.3313415662, & 
     1.3312995662,   1.3312575662,   1.3312155662,   1.3311735662,   1.3311305662, & 
     1.3310885662,   1.3310465662,   1.3310045662,   1.3309625662,   1.3309195662, & 
     1.3308775662,   1.3308355662,   1.3307935662,   1.3307515662,   1.3307085662, & 
     1.3306665662,   1.3306245662,   1.3305815662,   1.3305395662,   1.3304965662, & 
     1.3304535662,   1.3304105662,   1.3303675662,   1.3303255662,   1.3302825662, & 
     1.3302395662,   1.3301955662,   1.3301525662,   1.3301085662,   1.3300655662, & 
     1.3300215662,   1.3299775662,   1.3299335662,   1.3298895662,   1.3298445662, & 
     1.3297995662,   1.3297545662,   1.3297095662,   1.3296645662,   1.3296195662, & 
     1.3295745662,   1.3295275662,   1.3294825662,   1.3294365662,   1.3293895662, & 
     1.3293435662,   1.3292965662,   1.3292505662,   1.3292035662,   1.3291555662, & 
     1.3291085662,   1.3290605662,   1.3290125662,   1.3289645662,   1.3289155662, & 
     1.3288675662,   1.3288175662,   1.3287685662,   1.3287195662,   1.3286685662, & 
     1.3286195662,   1.3285685662,   1.3285185662,   1.3284695662,   1.3284205662, & 
     1.3283705662,   1.3283235662,   1.3282745662,   1.3282275662,   1.3281815662, & 
     1.3281345662,   1.3280925662,   1.3280465662,   1.3280045662,   1.3279615662, & 
     1.3279175662,   1.3278775662,   1.3278365662,   1.3277955662,   1.3277565662, & 
     1.3277165662,   1.3276765662,   1.3276375662,   1.3275975662,   1.3275565662, & 
     1.3275175662,   1.3274745662,   1.3274335662,   1.3273905662,   1.3273455662, & 
     1.3273035662,   1.3272555662,   1.3272115662,   1.3271635662,   1.3271145662, & 
     1.3270675662,   1.3270155662,   1.3269675662,   1.3269155662,   1.3268635662, & 
     1.3268125662,   1.3267585662,   1.3267065662,   1.3266515662,   1.3265965662, & 
     1.3265425662,   1.3264865662,   1.3264315662,   1.3263755662,   1.3263195662, & 
     1.3262635662,   1.3262065662,   1.3261505662,   1.3260925662,   1.3260345662, & 
     1.3259775662,   1.3259185662,   1.3258605662,   1.3258015662,   1.3257425662, & 
     1.3256835662,   1.3256235662,   1.3255645662,   1.3255055662,   1.3254465662, & 
     1.3253865662,   1.3253255662,   1.3252645662,   1.3252025662,   1.3251415662, & 
     1.3250795662,   1.3250175662,   1.3249565662,   1.3248935662,   1.3248295662, & 
     1.3247705662,   1.3247055662,   1.3246355662,   1.3245495662,   1.3244615662, & 
     1.3243735662,   1.3242835662,   1.3241935662,   1.3241025662,   1.3240105662, & 
     1.3239175662,   1.3238235662,   1.3237285662,   1.3236325662,   1.3235365662, & 
     1.3234385662,   1.3233395662,   1.3232405662,   1.3231395662,   1.3230375662, & 
     1.3229355662,   1.3228315662,   1.3227275662,   1.3226225662,   1.3225175662, & 
     1.3224095662,   1.3223015662,   1.3221935662,   1.3220865662,   1.3219755662, & 
     1.3218655662,   1.3217525662,   1.3216385662,   1.3215225662,   1.3214045662, & 
     1.3212835662,   1.3211615662,   1.3210375662,   1.3209115662,   1.3207835662, & 
     1.3206965662,   1.3205825662,   1.3204485662,   1.3202815662,   1.3201065662, & 
     1.3199615662,   1.3198175662,   1.3196285662,   1.3194055662,   1.3192525662, & 
     1.3190855662,   1.3189215662,   1.3187275662,   1.3184965662,   1.3182725662, & 
     1.3180655662,   1.3178685662,   1.3176895662,   1.3175515662,   1.3174505662, & 
     1.3173925662,   1.3173945662,   1.3174205662,   1.3174325662,   1.3174725662, & 
     1.3174875662,   1.3174835662,   1.3174775662,   1.3174605662,   1.3174685662, & 
     1.3174795662,   1.3174675662,   1.3174365662,   1.3174015662,   1.3173315662, & 
     1.3172365662,   1.3171585662,   1.3170715662,   1.3169735662,   1.3168655662, & 
     1.3167445662,   1.3166155662,   1.3164875662,   1.3163405662,   1.3162015662, & 
     1.3160575662,   1.3158995662,   1.3157355662,   1.3155855662,   1.3154175662, & 
     1.3152425662,   1.3150685662,   1.3149005662,   1.3147265662,   1.3145475662, & 
     1.3143635662,   1.3141845662,   1.3140055662,   1.3138275662,   1.3136355662, & 
     1.3134315662,   1.3132325662,   1.3130455662,   1.3128825662,   1.3127165662, & 
     1.3125105662,   1.3123005662,   1.3120915662,   1.3119065662,   1.3117235662, & 
     1.3115275662,   1.3113185662,   1.3111035662,   1.3108795662,   1.3106615662, & 
     1.3104465662,   1.3102265662,   1.3100055662,   1.3097845662,   1.3095705662, & 
     1.3093535662,   1.3091185662,   1.3088765662,   1.3086315662,   1.3083925662, & 
     1.3081575662,   1.3079215662,   1.3076745662,   1.3074315662,   1.3071825662, & 
     1.3069235662,   1.3066665662,   1.3064085662,   1.3061465662,   1.3058965662, & 
     1.3056375662,   1.3053825662,   1.3051115662,   1.3048335662,   1.3045525662, & 
     1.3042805662,   1.3040105662,   1.3037305662,   1.3034475662,   1.3031555662, & 
     1.3028605662,   1.3025685662,   1.3022755662,   1.3019695662,   1.3016575662, & 
     1.3013435662,   1.3010295662,   1.3007225662,   1.3004065662,   1.3000845662, & 
     1.2997535662,   1.2994185662,   1.2990885662,   1.2987485662,   1.2984105662, & 
     1.2980615662,   1.2977135662,   1.2973605662,   1.2969955662,   1.2966205662, & 
     1.2962515662,   1.2958785662,   1.2955015662,   1.2951165662,   1.2947335662, & 
     1.2943425662,   1.2939435662,   1.2935415662,   1.2931325662,   1.2927195662, & 
     1.2923045662,   1.2918835662,   1.2914565662,   1.2910235662,   1.2905805662, & 
     1.2901315662,   1.2896745662,   1.2892205662,   1.2887585662,   1.2882885662, & 
     1.2878065662,   1.2873155662,   1.2868235662,   1.2863235662,   1.2858185662, & 
     1.2852985662,   1.2847645662,   1.2842265662,   1.2836915662,   1.2831485662, & 
     1.2825905662,   1.2820275662,   1.2814545662,   1.2808755662,   1.2802825662, & 
     1.2796805662,   1.2790735662,   1.2784545662,   1.2778285662,   1.2771995662, & 
     1.2765545662,   1.2759085662,   1.2752535662,   1.2745765662,   1.2738955662, & 
     1.2732135662,   1.2725035662,   1.2717685662,   1.2710345662,   1.2702785662, & 
     1.2695125662,   1.2687325662,   1.2679305662,   1.2670145662,   1.2661025662, & 
     1.2652225662,   1.2643075662,   1.2633615662,   1.2623965662,   1.2614025662, & 
     1.2603805662,   1.2593285662,   1.2582505662,   1.2571475662,   1.2560025662, & 
     1.2548085662,   1.2535805662,   1.2523135662,   1.2510015662,   1.2496595662, & 
     1.2482715662,   1.2468475662,   1.2453765662,   1.2438485662,   1.2422735662, & 
     1.2406425662,   1.2389515662,   1.2371945662,   1.2353755662,   1.2334875662, & 
     1.2315245662,   1.2294765662,   1.2273265662,   1.2250845662,   1.2227265662, & 
     1.2202455662,   1.2176205662,   1.2148405662,   1.2118925662,   1.2087525662, & 
     1.2054235662,   1.2018565662,   1.1979945662,   1.1939065662,   1.1895245662, & 
     1.1848595662,   1.1800075662,   1.1749225662,   1.1697335662,   1.1646195662, & 
     1.1597135662,   1.1552255662,   1.1513295662,   1.1480585662,   1.1454875662, & 
     1.1435365662,   1.1421075662,   1.1410765662,   1.1403225662,   1.1397335662, & 
     1.1392585662,   1.1388595662,   1.1385195662,   1.1382285662,   1.1379875662, & 
     1.1378265662,   1.1377855662,   1.1379015662,   1.1381695662,   1.1386205662, & 
     1.1392915662,   1.1402115662,   1.1413845662,   1.1428255662,   1.1445345662, & 
     1.1465355662,   1.1488495662,   1.1514635662,   1.1543335662,   1.1574695662, & 
     1.1609055662,   1.1646285662,   1.1686085662,   1.1727805662,   1.1771595662, & 
     1.1818025662,   1.1867295662,   1.1918615662,   1.1972445662,   1.2028475662, & 
     1.2086795662,   1.2148105662,   1.2211325662,   1.2276115662,   1.2343675662, & 
     1.2413295662,   1.2485615662,   1.2560505662,   1.2636355662,   1.2712745662, & 
     1.2790765662,   1.2870315662,   1.2950895662,   1.3031215662,   1.3110535662, & 
     1.3188735662,   1.3265655662,   1.3341265662,   1.3414715662,   1.3484475662, & 
     1.3550775662,   1.3615155662,   1.3677525662,   1.3736085662,   1.3790545662, & 
     1.3841475662,   1.3889865662,   1.3936875662,   1.3982065662,   1.4025215662, & 
     1.4065645662,   1.4103125662,   1.4138525662,   1.4172995662,   1.4207065662, & 
     1.4240275662,   1.4271955662,   1.4302805662,   1.4333135662,   1.4363665662, & 
     1.4394365662,   1.4424365662,   1.4454015662,   1.4484085662,   1.4515125662, & 
     1.4546635662,   1.4577305662,   1.4607135662,   1.4636845662,   1.4666105662, & 
     1.4695115662,   1.4724035662,   1.4752665662,   1.4780405662,   1.4806235662, & 
     1.4830155662,   1.4852815662,   1.4874325662,   1.4893995662,   1.4911395662, & 
     1.4926565662,   1.4939915662,   1.4951635662,   1.4961595662,   1.4969665662, & 
     1.4975805662,   1.4980195662,   1.4983025662,   1.4984535662,   1.4984855662, & 
     1.4984045662,   1.4981975662,   1.4978625662,   1.4974085662,   1.4968595662, & 
     1.4962355662,   1.4955385662,   1.4947675662,   1.4939205662,   1.4930055662, & 
     1.4920315662,   1.4910105662,   1.4899455662,   1.4888475662,   1.4877165662, & 
     1.4865495662,   1.4853505662,   1.4841315662,   1.4829055662,   1.4816675662, & 
     1.4804075662,   1.4791365662,   1.4778585662,   1.4765725662,   1.4752785662, & 
     1.4739835662,   1.4726915662,   1.4713985662,   1.4701025662,   1.4688135662, & 
     1.4675285662,   1.4662485662,   1.4649765662,   1.4637115662,   1.4624505662, & 
     1.4611975662,   1.4599505662,   1.4587155662,   1.4574915662,   1.4562765662, & 
     1.4550725662,   1.4538785662,   1.4526935662,   1.4515185662,   1.4503555662, & 
     1.4492075662,   1.4480725662,   1.4469445662,   1.4458285662,   1.4447195662, & 
     1.4436235662,   1.4425355662,   1.4414545662,   1.4403915662,   1.4393445662, & 
     1.4383105662,   1.4372885662,   1.4362875662,   1.4352985662,   1.4343175662, & 
     1.4333495662,   1.4323955662,   1.4314495662,   1.4305145662,   1.4295865662, & 
     1.4286675662,   1.4277585662,   1.4268615662,   1.4259745662,   1.4251015662, & 
     1.4242425662,   1.4234005662,   1.4225725662,   1.4217565662,   1.4209465662, & 
     1.4201435662,   1.4193495662,   1.4185675662,   1.4177935662,   1.4170285662, & 
     1.4162735662,   1.4155295662,   1.4147955662,   1.4140675662,   1.4133515662, & 
     1.4126435662,   1.4119435662,   1.4112515662,   1.4105705662,   1.4099005662, & 
     1.4092355662,   1.4085755662,   1.4079295662,   1.4072985662,   1.4066725662, & 
     1.4060485662,   1.4054295662,   1.4048195662,   1.4042175662,   1.4036225662, & 
     1.4030355662,   1.4024555662,   1.4018835662,   1.4013175662,   1.4007575662, & 
     1.4002055662,   1.3996605662,   1.3991215662,   1.3985875662,   1.3980605662, & 
     1.3975395662,   1.3970235662,   1.3965125662,   1.3960075662,   1.3955095662, & 
     1.3950145662,   1.3945255662,   1.3940435662,   1.3935665662,   1.3930935662, & 
     1.3926255662,   1.3921615662,   1.3917025662,   1.3912505662,   1.3908015662, & 
     1.3903575662,   1.3899205662,   1.3894885662,   1.3890595662,   1.3886335662, & 
     1.3882115662,   1.3877965662,   1.3873845662,   1.3869765662,   1.3865715662, & 
     1.3861705662,   1.3857715662,   1.3853765662,   1.3849845662,   1.3845955662, & 
     1.3842105662,   1.3838305662,   1.3834525662,   1.3830775662,   1.3827055662, & 
     1.3823375662,   1.3819715662,   1.3816085662,   1.3812485662,   1.3808935662, & 
     1.3805405662,   1.3801915662,   1.3798465662,   1.3795035662,   1.3791635662, & 
     1.3788235662,   1.3784875662,   1.3781555662,   1.3778265662,   1.3775015662, & 
     1.3771795662,   1.3768595662,   1.3765405662,   1.3762235662,   1.3759085662, & 
     1.3755955662,   1.3752855662,   1.3749785662,   1.3746745662,   1.3743715662, & 
     1.3740715662,   1.3737745662,   1.3734785662/)
     
     

! 'abs_sw': absorption coefficients of sea water

     real(8), parameter, dimension(913)::abs_sw = (/0.7460000000, &
           0.4978283443,         0.3413501721,         0.2528978279,         0.2088036557, & 
           0.1854000000,         0.1638132607,         0.1443460607,         0.1317876508,         0.1238564410, & 
           0.1112000000,         0.0882043450,         0.0654389971,         0.0562046545,         0.0531626303, & 
           0.0515000000,         0.0497221697,         0.0463345455,         0.0399283975,         0.0310698738, & 
           0.0223000000,         0.0156944900,         0.0111243543,         0.0079241909,         0.0057681634, & 
           0.0046700000,         0.0035600000,         0.0027500000,         0.0020500000,         0.0017600000, & 
           0.0014700000,         0.0012400000,         0.0011200000,         0.0010100000,         0.0009500000, & 
           0.0008500000,         0.0008100000,         0.0008400000,         0.0009400000,         0.0009800000, & 
           0.0010600000,         0.0012000000,         0.0012200000,         0.0012700000,         0.0013300000, & 
           0.0014300000,         0.0015100000,         0.0016200000,         0.0017500000,         0.0019600000, & 
           0.0022200000,         0.0024800000,         0.0025900000,         0.0027100000,         0.0028800000, & 
           0.0031200000,         0.0033100000,         0.0035800000,         0.0039500000,         0.0044200000, & 
           0.0052200000,         0.0062600000,         0.0075312100,         0.0083804500,         0.0086688500, & 
           0.0090606350,         0.0094719500,         0.0100523000,         0.0106498408,         0.0114507000, & 
           0.0122121000,         0.0127450500,         0.0140556000,         0.0153982000,         0.0176248500, & 
           0.0206176500,         0.0242641850,         0.0298699500,         0.0363799500,         0.0411000000, & 
           0.0423005000,         0.0430805000,         0.0444370000,         0.0459505000,         0.0476005000, & 
           0.0494710000,         0.0519834250,         0.0561375000,         0.0599969500,         0.0621062500, & 
           0.0640380000,         0.0660023500,         0.0692688500,         0.0742209500,         0.0815683000, & 
           0.0926290000,         0.1083425000,         0.1276595000,         0.1532095000,         0.1869070000, & 
           0.2337345000,         0.2519090000,         0.2636665000,         0.2673050000,         0.2702050000, & 
           0.2774040000,         0.2839010000,         0.2920345000,         0.2984345000,         0.3047980000, & 
           0.3118920000,         0.3247250000,         0.3357525000,         0.3526160000,         0.3782071500, & 
           0.4094715000,         0.4268447500,         0.4363433500,         0.4452072000,         0.4522666500, & 
           0.4641285000,         0.4805230000,         0.4996480000,         0.5272400000,         0.5653603300, & 
           0.6066831800,         0.6870596700,         0.7816156150,         0.9096997500,         1.0668866650, & 
           1.2730176300,         1.4927886650,         1.7762506650,         2.1230815500,         2.3236907600, & 
           2.4751965000,         2.5716231250,         2.6177919250,         2.6396510700,         2.6470621850, & 
           2.6321898600,         2.6083176250,         2.5403627000,         2.4624320550,         2.3857269250, & 
           2.2805016750,         2.1747807750,         2.0921905000,         2.0198150000,         1.9753144100, & 
           1.9536781500,         1.9635113650,         2.0257926650,         2.1271796300,         2.2586725550, & 
           2.4561742100,         2.7420009900,         2.9423849050,         3.1394578850,         3.3443784450, & 
           3.6674216000,         3.9297686100,         4.2476380000,         4.4803666300,         4.6941159400, & 
           4.9368600000,         5.0965272500,         5.2601055000,         5.4310577200,         5.6203367800, & 
           5.7836927650,         5.9186245400,         6.1039979650,         6.2799156850,         6.4656809650, & 
           6.7046638550,         6.9356285150,         7.4114260100,         8.4361691250,         9.8083021400, & 
          11.2220438100,        13.6646274900,        17.3403992450,        20.9926081800,        24.6750100450, & 
          28.8292563200,        33.8240549650,        36.4545828800,        39.1427021450,        41.9914821850, & 
          43.8089924500,        44.6367670800,        45.2043043450,        45.3358827950,        44.8062014150, & 
          44.0134348600,        42.7492911100,        41.9425938950,        40.8813767350,        39.1574317700, & 
          37.6290768600,        35.6606047450,        33.7292745350,        32.0726286050,        30.5255589000, & 
          28.9801666300,        27.0695766550,        25.0404278250,        23.2310424450,        21.6422546250, & 
          20.0574236600,        18.7002442000,        17.3489391750,        16.3371666650,        15.6632818000, & 
          15.1118530950,        14.8882362450,        14.6572941750,        14.8095571900,        15.0836944600, & 
          15.4704674000,        16.1944132250,        16.9095687650,        17.7503911900,        18.6282191900, & 
          19.6544339250,        21.0730560600,        22.4902141100,        24.7707671400,        27.3435151850, & 
          30.4670148150,        35.3343274350,        40.1452280000,        46.9228172050,        55.6505121550, & 
          64.3814445000,        77.3869762950,        90.6899124600,       102.7845505650,       110.4903032150, & 
         116.6922557900,       118.7052298200,       120.5810784000,       121.8603880300,       122.9335402350, & 
         123.9579740150,       124.2243826800,       124.4661273000,       124.7488710050,       125.1847985950, & 
         125.2917632350,       124.4999350400,       123.6784978600,       122.7615902300,       121.5955104800, & 
         120.3833694550,       119.1099478150,       117.8437937850,       121.9220803650,       120.7637820000, & 
         118.4142165500,       115.6837263200,       112.6084456700,       111.4499944650,       110.4479818800, & 
         108.9665175150,       108.7978156550,       107.7503085450,       107.3028808000,       109.6357819800, & 
         111.2788172200,       114.8834061150,       117.6204459600,       121.4286050900,       126.3966818950, & 
         132.0819006200,       140.4359759000,       148.9178784950,       159.2979941050,       170.7317081050, & 
         181.3016815150,       195.4610302600,       210.8714009000,       229.2734422350,       250.9539299000, & 
         271.5542933050,       304.1080231000,       335.8553020050,       354.8679668950,       370.8438393100, & 
         387.2708850350,       417.5265264400,       454.7447842550,       501.7824959100,       571.4684542100, & 
         657.6495507100,       772.7141449450,       914.2614495900,      1157.2712352350,      1402.5296019800, & 
        1679.6488691800,      1945.6648059800,      2172.3938603950,      2383.2671160550,      2552.5686860200, & 
        2686.9829426650,      2795.3928101250,      2881.5669408700,      2943.6243850250,      2988.2561312950, & 
        3016.6803575900,      3027.5404200000,      3129.6797015500,      3125.6048607100,      3108.7912262850, & 
        3079.0868652400,      3017.9345602000,      2930.2335712650,      2834.3978910700,      2723.8317324750, & 
        2599.7819541850,      2485.7700453550,      2376.8581744350,      2243.3135656850,      2131.9043323850, & 
        2019.5166188050,      1916.4176148950,      1818.9350474350,      1720.2441727450,      1631.0614930150, & 
        1546.3186160600,      1467.4744671050,      1404.8132644500,      1340.9879594100,      1278.2249205600, & 
        1212.9250978400,      1147.4741700000,      1099.0290433300,      1057.0121374300,      1009.9334194300, & 
         970.2597828900,       933.1008555450,       897.5995170550,       866.0691144750,       837.9369607450, & 
         812.1766213550,       785.9055686050,       761.8489407850,       737.5761108700,       717.0820408800, & 
         698.7482728700,       681.7562811800,       666.9355803750,       653.7336307900,       640.7271944200, & 
         628.5434859200,       615.6007738850,       605.7194467000,       596.1526168600,       586.9972378600, & 
         579.2918954200,       572.0137746150,       564.6288872700,       557.6557234550,       552.1429718700, & 
         548.5868875900,       545.6283484000,       547.5995407900,       545.0687519500,       543.0971746900, & 
         542.1317128150,       542.3817829400,       542.8144810050,       543.4042218200,       545.4545110050, & 
         548.7677332550,       552.5727239200,       557.5957041350,       564.6069111950,       572.6282471200, & 
         582.4185222350,       592.3158255350,       603.9999929300,       616.5435223650,       631.1358723300, & 
         648.9440772200,       668.6818914750,       690.6740213800,       713.3411052150,       736.1979124200, & 
         759.4149595050,       787.0048443000,       813.3044465100,       833.9925325150,       852.5055177350, & 
         868.1254613050,       881.2617523000,       890.5882236100,       895.5272032950,       896.9423376950, & 
         898.2668950000,       902.3240886650,       900.8801143800,       899.3245373300,       900.5711443450, & 
         904.0786217750,       901.0629658600,       907.0067677500,       919.4810029450,       938.0892087000, & 
         962.5570678300,       993.0012818100,      1043.5878837100,      1111.3527593250,      1280.4100699200, & 
        1449.3015816550,      1634.9203483450,      1840.3701266550,      2202.9199516550,      2761.1164566550, & 
        3465.2561433450,      4411.1526516550,      5561.5576750000,      6854.4986650000,      8225.5532966550, & 
        9604.3389566550,     10767.0727800000,     11693.0435366550,     12369.6402933450,     12882.4808166550, & 
       13161.0713300000,     13390.5509266550,     13466.3130966550,     13508.4268150000,     13456.1340500000, & 
       13256.0620450000,     12972.1213033450,     12639.4818116550,     12241.9858783450,     11787.7135516550, & 
       11371.9918266550,     10971.6727333450,     10525.5026316550,     10109.4073216550,      9662.5786516550, & 
        9235.0862716550,      8832.8711733450,      8442.7263100000,      8055.7624200000,      7723.3863283450, & 
        7373.6448166550,      7030.2668300000,      6746.2507366550,      6472.2413300000,      6189.8571116550, & 
        5950.5349616550,      5716.4692316550,      5506.9588500000,      5299.5855466550,      5089.7642900000, & 
        4931.7169716550,      4769.1196883450,      4626.0313400000,      4460.5589366550,      4119.9685266550, & 
        3971.8911216550,      3821.7589500000,      3683.9668550000,      3563.3225883450,      3445.0240833450, & 
        3333.7441533450,      3230.7427150000,      3129.2147266550,      3032.4508616550,      2940.1025033450, & 
        2849.9423933450,      2768.3994466550,      2693.1035266550,      2622.3920116550,      2558.1941616550, & 
        2494.2260383450,      2435.6545850000,      2380.9178350000,      2330.7275900000,      2281.8375333450, & 
        2239.9466066550,      2199.0614233450,      2161.6429700000,      2128.1227053250,      2096.9220330950, & 
        2069.1419367200,      2044.4578161800,      2022.1293343600,      2001.2584650350,      1982.6124914550, & 
        1965.7907329200,      1949.6593410050,      1934.4618345650,      1921.6449869350,      1917.7063229200, & 
        1913.6685806850,      1911.7370740150,      1909.1861995100,      1905.5622425950,      1909.2245627550, & 
        1913.2241870650,      1918.5311071150,      1928.0635076350,      1937.3214232250,      1948.8336469000, & 
        1964.4696202300,      1981.8298862500,      2001.1382014850,      2020.7702026500,      2048.4723856850, & 
        2073.9411959450,      2105.0894191150,      2137.1600402000,      2167.9734483900,      2208.4869249050, & 
        2247.4437101600,      2288.3811263500,      2338.0972095900,      2387.6614097100,      2437.4027170850, & 
        2492.1766073350,      2544.7275336050,      2608.4275294400,      2671.0160120050,      2739.0663261750, & 
        2806.9483627700,      2881.7209658550,      2960.1330070100,      3033.9541453600,      3116.0543066350, & 
        3220.0881822700,      3321.6898815350,      3426.3392180350,      3504.9314586950,      3578.7258859600, & 
        3668.8358904150,      3775.0460681150,      3879.8294028550,      3991.6825947600,      4105.0789628200, & 
        4222.5216536450,      4344.3619222750,      4466.7473261650,      4591.4841936150,      4721.4490531950, & 
        4853.4526650300,      4986.5049031950,      5125.7697931950,      5267.6986625350,      5418.0259039000, & 
        5579.6079618100,      5743.3137333600,      5919.0587524950,      6088.3528616550,      6255.6261416550, & 
        6433.0971616550,      6644.2858766550,      6846.0888316550,      7057.3394658450,      7307.3648420950, & 
        7543.1250158450,      7778.7741570950,      8018.9429179050,      8245.4134316550,      8486.8725912500, & 
        8722.6372900000,      8944.9350737500,      9193.3491250000,      9419.2427104050,      9603.0966858450, & 
        9789.9189725000,     10206.4740287500,     10352.9254416550,     10492.0217191550,     10623.6838812500, & 
       10720.9489308450,     10806.5175566550,     10854.1759320950,     11032.1074591550,     11063.6036466550, & 
       11069.6732620950,     11061.0078625000,     11089.6745425000,     11124.3060500000,     11146.9488750000, & 
       11188.1047525000,     11240.1290825000,     11298.9164050000,     11373.3238325000,     11414.2788425000, & 
       11501.4322675000,     11623.1938225000,     11776.9967500000,     11974.2113675000,     12204.9195850000, & 
       12464.4711425000,     12748.7119025000,     13062.6525450000,     13440.4371325000,     13878.3722950000, & 
       14321.1783350000,     14798.3858725000,     15302.0684225000,     15847.4570025000,     16482.5974375000, & 
       17145.1798075000,     17842.7329275000,     18643.3869925000,     19576.4652575000,     20558.5964675000, & 
       21622.9492450000,     22935.0020391950,     24430.6845969450,     26328.1476669550,     28563.6045893150, & 
       31463.0558614850,     34396.1734521750,     37931.8201920000,     42362.4924251500,     47996.4229223500, & 
       54795.8274137500,     63203.1629121500,     73924.8642142000,     88398.4669383500,    106381.4895100000, & 
      129511.5287475000,    157220.0659049000,    187064.6771139000,    217469.1640910000,    248884.7314349000, & 
      282491.7479061490,    314530.7370659500,    342872.5217420500,    368252.0968284000,    392839.6547279500, & 
      416303.8722582250,    438809.5468891500,    460843.9729377500,    483204.5984639500,    505804.1750936000, & 
      529608.5295136990,    554401.8996532999,    581196.8656239000,    608040.7291214990,    635206.4073995000, & 
      663406.0497160000,    691653.3645194990,    720443.1335544990,    747855.4534135000,    774429.5144660000, & 
      801838.6145645001,    829371.8466850000,    853278.2675315000,    875907.0551835001,    898802.7509214990, & 
      921330.1900795000,    944157.8791530000,    966356.4403835000,    988853.1735285000,   1011051.6345810000, & 
     1034917.6540380000,   1057445.0634170000,   1080253.2258470000,   1099954.9448475000,   1122034.2733110001, & 
     1140600.3749915001,   1157096.2446544899,   1174691.4846355000,   1189406.6443270000,   1202469.3452049899, & 
     1213609.3612164999,   1224148.2822910000,   1234525.4265695000,   1241592.1132179899,   1247298.9148269901, & 
     1250963.4223674999,   1253287.6513836000,   1257143.7600481401,   1256891.2974095000,   1256803.4416910501, & 
     1251268.5215401601,   1243291.4861753399,   1237153.4722366999,   1228475.6029700399,   1222579.7602700000, & 
     1213489.0286425001,   1203952.7038375000,   1192032.2856689901,   1177505.5091240001,   1163217.5066104999, & 
     1148980.0239550001,   1133866.5349695000,   1119105.1585525000,   1102698.6651355000,   1087883.3521300000, & 
     1070927.9662830001,   1056454.7550844899,   1043316.8499695000,   1030724.7423785001,   1017996.6543415000, & 
     1003539.2159355000,    991131.1563790001,    978154.9026230000,    965516.5921360001,    955414.9974265000, & 
      944389.0708750000,    931860.4816225000,    919808.4413660000,    908530.1315265000,    898133.5539035000, & 
      886192.3827760000,    872952.2657005000,    860290.5398085000,    845941.0191220000,    830582.1295760000, & 
      815357.2455550000,    799578.8308275000,    785018.7426970000,    770029.7406320000,    754331.8316600000, & 
      739063.2043015000,    723078.7071270000,    706607.4956474990,    690395.7721275000,    673300.7699114990, & 
      656889.4969980000,    640135.0965374990,    622652.7897555000,    605668.9668760000,    587845.1362700000, & 
      570873.5114899999,    553231.4198039999,    535677.0025645000,    518358.2817839990,    501426.5545320000, & 
      485138.6301315000,    468515.2278210000,    453116.9989435000,    438525.0771450000,    424477.9381040000, & 
      411149.2145115000,    397782.3385140000,    385803.7980865000,    373880.3829070000,    362571.5013625000, & 
      351333.7135000000,    340321.6750590000,    330316.7296280000,    320267.8329710000,    310308.5845750000, & 
      300633.0690635000,    290594.0475920000,    280653.1527305000,    271044.1378370000,    261500.2197880000, & 
      252348.6176660000,    243229.0462310000,    233996.8867935000,    225181.6370635000,    217115.2671870000, & 
      209374.9177755000,    202381.1639350000,    195758.3041000000,    189556.7930405000,    183815.0118860000, & 
      178709.2335220000,    173659.2082375000,    169255.2207200000,    165149.9400680000,    161268.6664465000, & 
      157193.1971390000,    153230.3907530000,    148976.8593905000,    144631.0666655000,    140149.5958795000, & 
      135701.4357770000,    130920.2607185000,    126099.1618040000,    121145.8802865000,    116133.8284680000, & 
      109173.7518404000,    104681.4179278000,    100249.7698515500,     96342.7896175500,     92809.4394572500, & 
       89250.2206317500,     85434.3814163500,     81769.3446150000,     78197.2094884500,     75302.9288363000, & 
       73122.1478837000,     71346.5075957500,     69548.1338646000,     67740.9370079500,     65862.3437935000, & 
       64012.4027370500,     62056.1378743500,     60235.3839711500,     58389.7791864500,     56627.5854062500, & 
       54932.1563912500,     53312.8893308500,     51769.6447775500,     50306.5588239000,     48891.2988049500, & 
       47492.1317144500,     46152.8891703500,     44880.8846738500,     43651.4850085000,     42464.7982772500, & 
       41298.6269934500,     40171.5502266000,     39089.7642137500,     38026.3271795000,     37035.7252103000, & 
       36084.4635615500,     35116.1279752500,     34238.6934396000,     33346.3211832500,     32503.2705326000, & 
       31682.1948011500,     30906.8475705500,     30150.4620975500,     29411.4446591500,     28701.5306546500, & 
       28019.6855182000,     27365.1775063000,     26731.8804295500,     26134.0361913000,     25546.9679051000, & 
       24958.0140801000,     24397.6395054500,     23863.2751314000,     23325.0726002000,     22817.0520774000, & 
       22321.5685423000,     21835.2216019000,     21386.4258243500,     20952.0415672000,     20543.4505759000, & 
       20151.3646302500,     19770.7682782500,     19407.1558639500,     19051.8690244000,     18712.6120221500, & 
       18374.1712589500,     18049.8291506000,     17736.2392157500,     17443.9691635000,     17179.2822674500, & 
       16910.0557070500,     16647.2390582000,     16393.4386785000,     16141.6871698000,     15900.0740406000, & 
       15654.5766325000,     15424.3441517000,     15204.8855248500,     14991.4964385000,     14793.5846043000, & 
       14601.7844890000,     14408.8206837500,     14231.9395272500,     14045.6199748000,     13871.9834084000, & 
       13704.7170519000,     13554.8890632500,     13405.5447451500,     13254.2528453000,     13111.1920774000, & 
       12975.8311182500,     12857.2064637000,     12743.4560626500,     12629.0458020500,     12529.3475746000, & 
       12450.7212192500,     12362.0422425500,     12274.9177769500,     12197.7615734500,     12123.9156804000, & 
       12050.1457491000,     11981.5565335500,     11927.3228045000,     11883.8736697000,     11829.6623746500, & 
       11783.9342929500,     11737.7191100350,     11695.9406521700,     11651.5345834300,     11603.3515038650, & 
       11557.7190424550,     11515.6006936350,     11483.1208900900,     11453.7339796950,     11422.5498325550, & 
       11400.4753567900,     11377.5692310200,     11361.6315562600,     11352.3318961600,     11350.5719858600, & 
       11356.3227278400,     11365.3110176000,     11381.7211355250,     11402.8849408900,     11419.8797751800, &
       11437.0129028750,     11468.1714457600,     11503.5028219650/)
           
           

! 'sca_sw': scattering coefficients of pure sea water
    
     real(8), parameter, dimension(913)::sca_sw = (/0.1834336580, &
     0.1638590004,   0.1469535232,   0.1322776701,   0.1194758531, & 
     0.1082583696,   0.0983876210,   0.0896675184,   0.0819352736,   0.0750549892, & 
     0.0689126186,   0.0634119746,   0.0584715496,   0.0540219654,   0.0500039168, & 
     0.0463665044,   0.0430658751,   0.0400641079,   0.0373282974,   0.0348297952, & 
     0.0325435796,   0.0304477306,   0.0285229894,   0.0267523881,   0.0251209377, & 
     0.0236153633,   0.0222238796,   0.0209359987,   0.0197423669,   0.0186346231, & 
     0.0176052789,   0.0166476130,   0.0157555818,   0.0149237406,   0.0141471755, & 
     0.0134214439,   0.0127425228,   0.0121067629,   0.0115108494,   0.0109517661, & 
     0.0104267650,   0.0099333384,   0.0094691955,   0.0090322403,   0.0086205527, & 
     0.0082323716,   0.0078660800,   0.0075201910,   0.0071933361,   0.0068842546, & 
     0.0065917832,   0.0063148481,   0.0060524565,   0.0058036902,   0.0055676984, & 
     0.0053436928,   0.0051309419,   0.0049287664,   0.0047365348,   0.0045536600, & 
     0.0043795952,   0.0042138309,   0.0040558921,   0.0039053357,   0.0037617476, & 
     0.0036247410,   0.0034939543,   0.0033690488,   0.0032497075,   0.0031356334, & 
     0.0030265478,   0.0029221894,   0.0028223127,   0.0027266872,   0.0026350962, & 
     0.0025473358,   0.0024632142,   0.0023825507,   0.0023051752,   0.0022309272, & 
     0.0021596555,   0.0020912173,   0.0020254777,   0.0019623094,   0.0019015922, & 
     0.0018432122,   0.0017870618,   0.0017330392,   0.0016810479,   0.0016309966, & 
     0.0015827987,   0.0015363723,   0.0014916393,   0.0014485260,   0.0014069622, & 
     0.0013668814,   0.0013282204,   0.0012909189,   0.0012549200,   0.0012201691, & 
     0.0011866147,   0.0011542074,   0.0011229004,   0.0010926491,   0.0010634109, & 
     0.0010351453,   0.0010078136,   0.0009813790,   0.0009558063,   0.0009310619, & 
     0.0009071137,   0.0008839312,   0.0008614851,   0.0008397475,   0.0008186916, & 
     0.0007982920,   0.0007785241,   0.0007593647,   0.0007407912,   0.0007227825, & 
     0.0007053178,   0.0006883777,   0.0006719433,   0.0006559965,   0.0006405200, & 
     0.0006254974,   0.0006109127,   0.0005967507,   0.0005829967,   0.0005696367, & 
     0.0005566573,   0.0005440454,   0.0005317886,   0.0005198751,   0.0005082933, & 
     0.0004970322,   0.0004860812,   0.0004754301,   0.0004650690,   0.0004549887, & 
     0.0004451799,   0.0004356339,   0.0004263424,   0.0004172972,   0.0004084905, & 
     0.0003999149,   0.0003915630,   0.0003834280,   0.0003755030,   0.0003677817, & 
     0.0003602577,   0.0003529429,   0.0003458138,   0.0003388668,   0.0003320921, & 
     0.0003254869,   0.0003190459,   0.0003127625,   0.0003066319,   0.0003006517, & 
     0.0002948156,   0.0002891195,   0.0002835612,   0.0002781367,   0.0002728405, & 
     0.0002676689,   0.0002626184,   0.0002576874,   0.0002528708,   0.0002481655, & 
     0.0002435699,   0.0002390796,   0.0002346915,   0.0002304045,   0.0002262143, & 
     0.0002221184,   0.0002181141,   0.0002142004,   0.0002103722,   0.0002066298, & 
     0.0002029711,   0.0001993913,   0.0001958907,   0.0001924661,   0.0001891156, & 
     0.0001858372,   0.0001826302,   0.0001794917,   0.0001764187,   0.0001734119, & 
     0.0001704695,   0.0001675878,   0.0001647675,   0.0001620047,   0.0001593003, & 
     0.0001566517,   0.0001540576,   0.0001515166,   0.0001490274,   0.0001465888, & 
     0.0001441996,   0.0001418585,   0.0001395653,   0.0001373170,   0.0001351145, & 
     0.0001329556,   0.0001308393,   0.0001287640,   0.0001267301,   0.0001247359, & 
     0.0001227806,   0.0001208632,   0.0001189828,   0.0001171385,   0.0001153297, & 
     0.0001135553,   0.0001118141,   0.0001101065,   0.0001084312,   0.0001067874, & 
     0.0001051744,   0.0001035915,   0.0001020381,   0.0001005135,   0.0000990171, & 
     0.0000975482,   0.0000961062,   0.0000946906,   0.0000933008,   0.0000919362, & 
     0.0000905963,   0.0000892805,   0.0000879884,   0.0000867194,   0.0000854724, & 
     0.0000842482,   0.0000830456,   0.0000818643,   0.0000807037,   0.0000795630, & 
     0.0000784426,   0.0000773418,   0.0000762601,   0.0000751971,   0.0000741519, & 
     0.0000731252,   0.0000721160,   0.0000711236,   0.0000701485,   0.0000691895, & 
     0.0000682467,   0.0000673198,   0.0000664084,   0.0000655127,   0.0000646315, & 
     0.0000637650,   0.0000629124,   0.0000620743,   0.0000612496,   0.0000604388, & 
     0.0000596409,   0.0000588560,   0.0000580838,   0.0000573242,   0.0000565765, & 
     0.0000558408,   0.0000551170,   0.0000544047,   0.0000537038,   0.0000530141, & 
     0.0000523352,   0.0000516665,   0.0000510090,   0.0000503614,   0.0000497237, & 
     0.0000490963,   0.0000484783,   0.0000478703,   0.0000472714,   0.0000466814, & 
     0.0000461008,   0.0000455288,   0.0000449655,   0.0000444109,   0.0000438644, & 
     0.0000433265,   0.0000427962,   0.0000422741,   0.0000417599,   0.0000412529, & 
     0.0000407540,   0.0000402621,   0.0000397777,   0.0000393007,   0.0000388308, & 
     0.0000383676,   0.0000379119,   0.0000374625,   0.0000370200,   0.0000365843, & 
     0.0000361546,   0.0000357324,   0.0000353153,   0.0000349051,   0.0000345006, & 
     0.0000341016,   0.0000337092,   0.0000333222,   0.0000329406,   0.0000325649, & 
     0.0000321943,   0.0000318288,   0.0000314687,   0.0000311135,   0.0000307629, & 
     0.0000304177,   0.0000300765,   0.0000297403,   0.0000294085,   0.0000290808, & 
     0.0000287582,   0.0000284389,   0.0000281247,   0.0000278141,   0.0000275075, & 
     0.0000272054,   0.0000269065,   0.0000266124,   0.0000263216,   0.0000260347, & 
     0.0000257518,   0.0000254721,   0.0000251966,   0.0000249242,   0.0000246555, & 
     0.0000243905,   0.0000241286,   0.0000238704,   0.0000236155,   0.0000233638, & 
     0.0000231155,   0.0000228703,   0.0000226284,   0.0000223894,   0.0000221535, & 
     0.0000219208,   0.0000216908,   0.0000214639,   0.0000212398,   0.0000210185, & 
     0.0000208001,   0.0000205844,   0.0000203715,   0.0000201613,   0.0000199538, & 
     0.0000197488,   0.0000195463,   0.0000193463,   0.0000191487,   0.0000189536, & 
     0.0000187610,   0.0000185707,   0.0000183829,   0.0000181972,   0.0000180136, & 
     0.0000178330,   0.0000176538,   0.0000174763,   0.0000172992,   0.0000171241, & 
     0.0000169511,   0.0000167800,   0.0000166111,   0.0000164441,   0.0000162790, & 
     0.0000161159,   0.0000159546,   0.0000157952,   0.0000156377,   0.0000154820, & 
     0.0000153280,   0.0000151758,   0.0000150255,   0.0000148767,   0.0000147297, & 
     0.0000145844,   0.0000144406,   0.0000142986,   0.0000141582,   0.0000140194, & 
     0.0000138821,   0.0000137463,   0.0000136122,   0.0000134797,   0.0000133485, & 
     0.0000132189,   0.0000130906,   0.0000129636,   0.0000128380,   0.0000127138, & 
     0.0000125907,   0.0000124689,   0.0000123485,   0.0000122293,   0.0000121113, & 
     0.0000119978,   0.0000118836,   0.0000117692,   0.0000116536,   0.0000115387, & 
     0.0000114274,   0.0000113175,   0.0000112056,   0.0000110925,   0.0000109857, & 
     0.0000108793,   0.0000107742,   0.0000106683,   0.0000105610,   0.0000104555, & 
     0.0000103524,   0.0000102511,   0.0000101523,   0.0000100572,   0.0000099657, & 
     0.0000098779,   0.0000097949,   0.0000097143,   0.0000096337,   0.0000095556, & 
     0.0000094769,   0.0000093977,   0.0000093193,   0.0000092410,   0.0000091650, & 
     0.0000090900,   0.0000090143,   0.0000089384,   0.0000088630,   0.0000087864, & 
     0.0000087092,   0.0000086337,   0.0000085586,   0.0000084836,   0.0000084089, & 
     0.0000083342,   0.0000082600,   0.0000081865,   0.0000081129,   0.0000080404, & 
     0.0000079685,   0.0000078966,   0.0000078252,   0.0000077553,   0.0000076852, & 
     0.0000076155,   0.0000075466,   0.0000074787,   0.0000074113,   0.0000073444, & 
     0.0000072779,   0.0000072124,   0.0000071475,   0.0000070834,   0.0000070194, & 
     0.0000069555,   0.0000068925,   0.0000068307,   0.0000067706,   0.0000067110, & 
     0.0000066503,   0.0000065901,   0.0000065305,   0.0000064726,   0.0000064154, & 
     0.0000063583,   0.0000063012,   0.0000062445,   0.0000061880,   0.0000061323, & 
     0.0000060773,   0.0000060228,   0.0000059687,   0.0000059152,   0.0000058626, & 
     0.0000058104,   0.0000057580,   0.0000057060,   0.0000056543,   0.0000056035, & 
     0.0000055533,   0.0000055036,   0.0000054540,   0.0000054051,   0.0000053565, & 
     0.0000053080,   0.0000052601,   0.0000052127,   0.0000051657,   0.0000051195, & 
     0.0000050735,   0.0000050281,   0.0000049827,   0.0000049375,   0.0000048927, & 
     0.0000048486,   0.0000048051,   0.0000047617,   0.0000047187,   0.0000046758, & 
     0.0000046333,   0.0000045913,   0.0000045497,   0.0000045082,   0.0000044669, & 
     0.0000044260,   0.0000043855,   0.0000043456,   0.0000043059,   0.0000042665, & 
     0.0000042272,   0.0000041882,   0.0000041497,   0.0000041114,   0.0000040735, & 
     0.0000040357,   0.0000039984,   0.0000039613,   0.0000039243,   0.0000038874, & 
     0.0000038510,   0.0000038150,   0.0000037792,   0.0000037436,   0.0000037084, & 
     0.0000036734,   0.0000036386,   0.0000036040,   0.0000035697,   0.0000035356, & 
     0.0000035019,   0.0000034684,   0.0000034350,   0.0000034019,   0.0000033690, & 
     0.0000033362,   0.0000033037,   0.0000032715,   0.0000032395,   0.0000032077, & 
     0.0000031760,   0.0000031444,   0.0000031132,   0.0000030821,   0.0000030513, & 
     0.0000030204,   0.0000029897,   0.0000029592,   0.0000029291,   0.0000028992, & 
     0.0000028693,   0.0000028397,   0.0000028102,   0.0000027809,   0.0000027517, & 
     0.0000027227,   0.0000026939,   0.0000026652,   0.0000026367,   0.0000026085, & 
     0.0000025803,   0.0000025524,   0.0000025247,   0.0000024970,   0.0000024695, & 
     0.0000024423,   0.0000024150,   0.0000023875,   0.0000023605,   0.0000023334, & 
     0.0000023064,   0.0000022796,   0.0000022528,   0.0000022245,   0.0000021967, & 
     0.0000021697,   0.0000021426,   0.0000021153,   0.0000020882,   0.0000020610, & 
     0.0000020337,   0.0000020064,   0.0000019792,   0.0000019519,   0.0000019245, & 
     0.0000018969,   0.0000018692,   0.0000018414,   0.0000018135,   0.0000017856, & 
     0.0000017576,   0.0000017295,   0.0000017014,   0.0000016731,   0.0000016447, & 
     0.0000016161,   0.0000015874,   0.0000015585,   0.0000015294,   0.0000015001, & 
     0.0000014706,   0.0000014408,   0.0000014106,   0.0000013801,   0.0000013491, & 
     0.0000013177,   0.0000012856,   0.0000012530,   0.0000012197,   0.0000011856, & 
     0.0000011509,   0.0000011153,   0.0000010785,   0.0000010412,   0.0000010031, & 
     0.0000009643,   0.0000009258,   0.0000008874,   0.0000008499,   0.0000008145, & 
     0.0000007818,   0.0000007527,   0.0000007277,   0.0000007068,   0.0000006900, & 
     0.0000006765,   0.0000006658,   0.0000006571,   0.0000006498,   0.0000006433, & 
     0.0000006374,   0.0000006319,   0.0000006267,   0.0000006218,   0.0000006172, & 
     0.0000006129,   0.0000006092,   0.0000006062,   0.0000006038,   0.0000006022, & 
     0.0000006016,   0.0000006021,   0.0000006036,   0.0000006063,   0.0000006102, & 
     0.0000006154,   0.0000006220,   0.0000006301,   0.0000006395,   0.0000006502, & 
     0.0000006626,   0.0000006767,   0.0000006924,   0.0000007094,   0.0000007279, & 
     0.0000007483,   0.0000007709,   0.0000007953,   0.0000008219,   0.0000008507, & 
     0.0000008819,   0.0000009160,   0.0000009527,   0.0000009918,   0.0000010343, & 
     0.0000010799,   0.0000011294,   0.0000011830,   0.0000012396,   0.0000012991, & 
     0.0000013624,   0.0000014299,   0.0000015012,   0.0000015752,   0.0000016513, & 
     0.0000017291,   0.0000018086,   0.0000018895,   0.0000019707,   0.0000020500, & 
     0.0000021273,   0.0000022043,   0.0000022807,   0.0000023538,   0.0000024225, & 
     0.0000024875,   0.0000025499,   0.0000026114,   0.0000026711,   0.0000027285, & 
     0.0000027822,   0.0000028318,   0.0000028784,   0.0000029240,   0.0000029694, & 
     0.0000030139,   0.0000030561,   0.0000030972,   0.0000031378,   0.0000031792, & 
     0.0000032214,   0.0000032627,   0.0000033038,   0.0000033461,   0.0000033908, & 
     0.0000034370,   0.0000034819,   0.0000035257,   0.0000035696,   0.0000036130, & 
     0.0000036563,   0.0000036998,   0.0000037432,   0.0000037850,   0.0000038229, & 
     0.0000038568,   0.0000038881,   0.0000039170,   0.0000039419,   0.0000039616, & 
     0.0000039763,   0.0000039868,   0.0000039935,   0.0000039962,   0.0000039945, & 
     0.0000039884,   0.0000039782,   0.0000039645,   0.0000039479,   0.0000039286, & 
     0.0000039069,   0.0000038825,   0.0000038554,   0.0000038258,   0.0000037945, & 
     0.0000037617,   0.0000037277,   0.0000036924,   0.0000036559,   0.0000036183, & 
     0.0000035799,   0.0000035409,   0.0000035014,   0.0000034618,   0.0000034219, & 
     0.0000033818,   0.0000033415,   0.0000033014,   0.0000032616,   0.0000032220, & 
     0.0000031825,   0.0000031433,   0.0000031045,   0.0000030660,   0.0000030278, & 
     0.0000029901,   0.0000029529,   0.0000029162,   0.0000028799,   0.0000028441, & 
     0.0000028088,   0.0000027741,   0.0000027399,   0.0000027063,   0.0000026731, & 
     0.0000026405,   0.0000026084,   0.0000025768,   0.0000025458,   0.0000025153, & 
     0.0000024853,   0.0000024558,   0.0000024268,   0.0000023983,   0.0000023703, & 
     0.0000023429,   0.0000023159,   0.0000022893,   0.0000022633,   0.0000022376, & 
     0.0000022123,   0.0000021875,   0.0000021630,   0.0000021391,   0.0000021156, & 
     0.0000020925,   0.0000020699,   0.0000020477,   0.0000020260,   0.0000020045, & 
     0.0000019835,   0.0000019628,   0.0000019425,   0.0000019225,   0.0000019028, & 
     0.0000018834,   0.0000018643,   0.0000018455,   0.0000018271,   0.0000018090, & 
     0.0000017912,   0.0000017738,   0.0000017567,   0.0000017399,   0.0000017233, & 
     0.0000017070,   0.0000016909,   0.0000016751,   0.0000016596,   0.0000016442, & 
     0.0000016292,   0.0000016143,   0.0000015997,   0.0000015853,   0.0000015712, & 
     0.0000015572,   0.0000015435,   0.0000015300,   0.0000015167,   0.0000015036, & 
     0.0000014906,   0.0000014779,   0.0000014654,   0.0000014531,   0.0000014410, & 
     0.0000014289,   0.0000014171,   0.0000014054,   0.0000013939,   0.0000013826, & 
     0.0000013714,   0.0000013604,   0.0000013495,   0.0000013388,   0.0000013282, & 
     0.0000013177,   0.0000013075,   0.0000012973,   0.0000012873,   0.0000012774, & 
     0.0000012676,   0.0000012580,   0.0000012484,   0.0000012390,   0.0000012297, & 
     0.0000012205,   0.0000012115,   0.0000012025,   0.0000011937,   0.0000011850, & 
     0.0000011763,   0.0000011678,   0.0000011594,   0.0000011510,   0.0000011428, & 
     0.0000011347,   0.0000011267,   0.0000011187,   0.0000011109,   0.0000011031, & 
     0.0000010954,   0.0000010879,   0.0000010804,   0.0000010730,   0.0000010656, & 
     0.0000010584,   0.0000010512,   0.0000010441,   0.0000010370,   0.0000010300, & 
     0.0000010232,   0.0000010163,   0.0000010096,   0.0000010029,   0.0000009963, & 
     0.0000009897,   0.0000009833,   0.0000009768,   0.0000009705,   0.0000009642, & 
     0.0000009580,   0.0000009518,   0.0000009457,   0.0000009397,   0.0000009337, & 
     0.0000009278,   0.0000009219,   0.0000009161,   0.0000009104,   0.0000009047, & 
     0.0000008991,   0.0000008935,   0.0000008879,   0.0000008824,   0.0000008770, & 
     0.0000008716,   0.0000008663,   0.0000008610,   0.0000008558,   0.0000008506, &
     0.0000008454,   0.0000008403,   0.0000008353/)
     

! 'stand_solar': standard irradiance solar spectra
    
     real(8), parameter, dimension(913)::stand_solar = (/0.0069744917, &
     0.0098032593,   0.0134890579,   0.0328698564,   0.0335830888, & 
     0.0481106286,   0.0623513551,   0.0468597595,   0.0524332425,   0.0511567849, & 
     0.0415842862,   0.0640534878,   0.0500935913,   0.0414632222,   0.0901252732, & 
     0.0924310257,   0.2132787078,   0.2489245408,   0.2039947025,   0.2143602377, & 
     0.0751764030,   0.2861794434,   0.3143131167,   0.5444277045,   0.5355990571, & 
     0.4514268658,   0.6236254053,   0.6342517617,   0.6938163555,   0.6144212062, & 
     0.7769851383,   0.6989831230,   0.9150282334,   0.9617316548,   0.8724377984, & 
     0.9947137291,   0.8517756461,   0.9039866064,   0.9220385036,   0.9777080857, & 
     1.0254926290,   0.9895369342,   1.1186101038,   1.1331857040,   1.0731664668, & 
     1.1135338613,   0.8127026186,   0.9755366538,   1.2423832248,   1.1062589544, & 
     1.6666289265,   1.6700402919,   1.6824376941,   1.8393723605,   1.8064252408, & 
     1.7181243568,   1.7661999823,   1.5624222074,   1.7194582610,   1.8377304305, & 
     1.7802452542,   1.9500988792,   2.0381873016,   2.0159876368,   2.0552892370, & 
     2.0319993503,   1.9897097115,   2.0141846759,   2.0534924052,   1.9662169409, & 
     2.0575941321,   2.0030446825,   1.8946992681,   1.8903633204,   1.9484098875, & 
     1.9340298567,   1.8903823187,   1.9093899679,   1.9363964630,   1.7780238603, & 
     1.8396074327,   1.9309430844,   1.8828509437,   1.8485149990,   1.9468088594, & 
     1.7799037327,   1.8843902763,   1.8399348633,   1.8570981981,   1.8546155640, & 
     1.8320712183,   1.8635757904,   1.8453821175,   1.8675549953,   1.8309119650, & 
     1.8387412223,   1.8652601645,   1.8184806936,   1.7892443967,   1.7965572641, & 
     1.7602398473,   1.7904731170,   1.7482948516,   1.7337280874,   1.6509541105, & 
     1.7246193538,   1.6594505183,   1.6964061880,   1.6396859618,   1.6541122199, & 
     1.6317601164,   1.6265294103,   1.5731516274,   1.6099683734,   1.4360306005, & 
     1.5820850341,   1.5597848764,   1.5410282352,   1.5161104158,   1.5083925864, & 
     1.4945107940,   1.4645085290,   1.4671086121,   1.4475120402,   1.4228099512, & 
     1.4074522766,   1.4038969213,   1.3894796218,   1.3799427841,   1.3622193558, & 
     1.3255031976,   1.3500754112,   1.3313025592,   1.3244104175,   1.3095682859, & 
     1.2620455845,   1.2849448974,   1.2780579785,   1.2603668011,   1.2518272370, & 
     1.2372145175,   1.2355510082,   1.1911245977,   1.2076933596,   1.1908324369, & 
     1.1869607574,   1.1771181670,   1.1763185811,   1.1504669690,   1.1443439849, & 
     1.1370988197,   1.1190394902,   1.1064540434,   1.1181921168,   1.1094855925, & 
     1.0681061780,   1.0716258995,   1.0810000000,   1.0510000000,   1.0520000000, & 
     1.0400000000,   1.0240000000,   1.0230000000,   0.9961000000,   0.9755000000, & 
     0.9969000000,   0.9993000000,   0.9395000000,   0.9833000000,   0.9640000000, & 
     0.9554000000,   0.9579000000,   0.9443000000,   0.9394000000,   0.9387000000, & 
     0.9158000000,   0.9285000000,   0.9025000000,   0.8967000000,   0.9004000000, & 
     0.8908000000,   0.8585000000,   0.8769000000,   0.8651000000,   0.8547000000, & 
     0.8438000000,   0.8324000000,   0.8407000000,   0.8317000000,   0.8082000000, & 
     0.8151000000,   0.8013000000,   0.7969000000,   0.7936000000,   0.7825000000, & 
     0.7746000000,   0.7698000000,   0.7615000000,   0.7567000000,   0.7525000000, & 
     0.7479000000,   0.7261000000,   0.7335000000,   0.7262000000,   0.7139000000, & 
     0.7041000000,   0.7054000000,   0.6989000000,   0.6905000000,   0.6840000000, & 
     0.6761000000,   0.6714000000,   0.6644000000,   0.6586000000,   0.6557000000, & 
     0.6415000000,   0.6459000000,   0.6317000000,   0.6326000000,   0.6280000000, & 
     0.6247000000,   0.6180000000,   0.6103000000,   0.6159000000,   0.5983000000, & 
     0.6042000000,   0.5974000000,   0.5916000000,   0.5843000000,   0.5831000000, & 
     0.5741000000,   0.5769000000,   0.5700000000,   0.5678000000,   0.5658000000, & 
     0.5530000000,   0.5514000000,   0.5525000000,   0.5458000000,   0.5455000000, & 
     0.5320000000,   0.5332000000,   0.5312000000,   0.5315000000,   0.5184000000, & 
     0.5239000000,   0.5103000000,   0.5075000000,   0.5161000000,   0.5084000000, & 
     0.5003000000,   0.4948000000,   0.4890000000,   0.4933000000,   0.4930000000, & 
     0.4875000000,   0.4846000000,   0.4771000000,   0.4750000000,   0.4719000000, & 
     0.4653000000,   0.4619000000,   0.4633000000,   0.4571000000,   0.4551000000, & 
     0.4530000000,   0.4478000000,   0.4417000000,   0.4452000000,   0.4451000000, & 
     0.4356000000,   0.4259000000,   0.4314000000,   0.4254000000,   0.4224000000, & 
     0.4186000000,   0.4111000000,   0.4123000000,   0.4033000000,   0.3979000000, & 
     0.4016000000,   0.3981000000,   0.3908000000,   0.3863000000,   0.3866000000, & 
     0.3790000000,   0.3798000000,   0.3766000000,   0.3742000000,   0.3675000000, & 
     0.3673000000,   0.3657000000,   0.3628000000,   0.3621000000,   0.3561000000, & 
     0.3579000000,   0.3547000000,   0.3530000000,   0.3513000000,   0.3487000000, & 
     0.3427000000,   0.3428000000,   0.3427000000,   0.3348000000,   0.3385000000, & 
     0.3357000000,   0.3311000000,   0.3285000000,   0.3300000000,   0.3285000000, & 
     0.3188000000,   0.3197000000,   0.3216000000,   0.3154000000,   0.3131000000, & 
     0.3156000000,   0.3105000000,   0.3114000000,   0.3073000000,   0.3048000000, & 
     0.3068000000,   0.3039000000,   0.2855000000,   0.3018000000,   0.2972000000, & 
     0.3011000000,   0.2799000000,   0.2919000000,   0.2913000000,   0.2882000000, & 
     0.2866000000,   0.2835000000,   0.2846000000,   0.2823000000,   0.2806000000, & 
     0.2730000000,   0.2778000000,   0.2711000000,   0.2731000000,   0.2671000000, & 
     0.2683000000,   0.2669000000,   0.2633000000,   0.2673000000,   0.2536000000, & 
     0.2650000000,   0.2591000000,   0.2490000000,   0.2526000000,   0.2506000000, & 
     0.2512000000,   0.2497000000,   0.2491000000,   0.2430000000,   0.2374000000, & 
     0.2369000000,   0.2416000000,   0.2418000000,   0.2387000000,   0.2357000000, & 
     0.2262000000,   0.2278000000,   0.2292000000,   0.2268000000,   0.2260000000, & 
     0.2245000000,   0.2227000000,   0.2193000000,   0.2173000000,   0.2161000000, & 
     0.2080000000,   0.2129000000,   0.2120000000,   0.2123000000,   0.2100000000, & 
     0.2063000000,   0.2052000000,   0.2017000000,   0.1982000000,   0.2022000000, & 
     0.1993000000,   0.1954000000,   0.1971000000,   0.1936000000,   0.1827000000, & 
     0.1905000000,   0.1907000000,   0.1872000000,   0.1850000000,   0.1849000000, & 
     0.1831000000,   0.1807000000,   0.1802000000,   0.1794000000,   0.1763000000, & 
     0.1756000000,   0.1735000000,   0.1747000000,   0.1721000000,   0.1706000000, & 
     0.1699000000,   0.1688000000,   0.1685000000,   0.1675000000,   0.1605000000, & 
     0.1596000000,   0.1624000000,   0.1611000000,   0.1593000000,   0.1581000000, & 
     0.1562000000,   0.1541000000,   0.1510000000,   0.1534000000,   0.1509000000, & 
     0.1503000000,   0.1509000000,   0.1492000000,   0.1473000000,   0.1299000000, & 
     0.1462000000,   0.1464000000,   0.1453000000,   0.1408000000,   0.1373000000, & 
     0.1397000000,   0.1386000000,   0.1377000000,   0.1354000000,   0.1360000000, & 
     0.1333000000,   0.1341000000,   0.1322000000,   0.1308000000,   0.1328000000, & 
     0.1299000000,   0.1203000000,   0.1271000000,   0.1255000000,   0.1276000000, & 
     0.1261000000,   0.1222000000,   0.1240000000,   0.1213000000,   0.1224000000, & 
     0.1196000000,   0.1197000000,   0.1195000000,   0.1180000000,   0.1173000000, & 
     0.1170000000,   0.1148000000,   0.1154000000,   0.1145000000,   0.1137000000, & 
     0.1116000000,   0.1116000000,   0.1107000000,   0.1098000000,   0.1083000000, & 
     0.1078000000,   0.1076000000,   0.1063000000,   0.1047000000,   0.1046000000, & 
     0.1028000000,   0.1005000000,   0.1019000000,   0.1004000000,   0.1006000000, & 
     0.0987800000,   0.0977200000,   0.0983500000,   0.0956700000,   0.0958000000, & 
     0.0960600000,   0.0955900000,   0.0951300000,   0.0945200000,   0.0933100000, & 
     0.0927500000,   0.0918900000,   0.0922500000,   0.0921000000,   0.0901200000, & 
     0.0908300000,   0.0900600000,   0.0897900000,   0.0891300000,   0.0887400000, & 
     0.0878100000,   0.0845600000,   0.0830000000,   0.0859100000,   0.0853200000, & 
     0.0849700000,   0.0845700000,   0.0827700000,   0.0837700000,   0.0831800000, & 
     0.0826500000,   0.0821100000,   0.0796600000,   0.0810500000,   0.0799400000, & 
     0.0799700000,   0.0792600000,   0.0782600000,   0.0781500000,   0.0775800000, & 
     0.0763900000,   0.0762400000,   0.0752000000,   0.0751200000,   0.0742200000, & 
     0.0742100000,   0.0732900000,   0.0732700000,   0.0727700000,   0.0723900000, & 
     0.0716500000,   0.0712500000,   0.0712700000,   0.0706700000,   0.0690800000, & 
     0.0695300000,   0.0693100000,   0.0690100000,   0.0686700000,   0.0677900000, & 
     0.0676800000,   0.0653600000,   0.0662900000,   0.0658400000,   0.0653600000, & 
     0.0652000000,   0.0650900000,   0.0647200000,   0.0628900000,   0.0628200000, & 
     0.0630800000,   0.0629500000,   0.0626300000,   0.0620700000,   0.0616400000, & 
     0.0617200000,   0.0588500000,   0.0600400000,   0.0600800000,   0.0599600000, & 
     0.0594400000,   0.0594500000,   0.0591500000,   0.0589400000,   0.0559900000, & 
     0.0577000000,   0.0572600000,   0.0571200000,   0.0570200000,   0.0561800000, & 
     0.0563900000,   0.0560300000,   0.0545700000,   0.0543200000,   0.0537000000, & 
     0.0545700000,   0.0543500000,   0.0539000000,   0.0533000000,   0.0534300000, & 
     0.0517700000,   0.0521900000,   0.0516900000,   0.0519800000,   0.0515200000, & 
     0.0515500000,   0.0514064515,   0.0511141911,   0.0507227049,   0.0502814792, & 
     0.0498400000,   0.0494379870,   0.0490760936,   0.0487452067,   0.0484362132, & 
     0.0481400000,   0.0478489606,   0.0475615147,   0.0472775885,   0.0469971081, & 
     0.0467200000,   0.0464471306,   0.0461831277,   0.0459335595,   0.0457039942, & 
     0.0455000000,   0.0453232371,   0.0451597346,   0.0449916134,   0.0448009949, & 
     0.0445700000,   0.0442876810,   0.0439708141,   0.0436431067,   0.0433282662, & 
     0.0430500000,   0.0428245991,   0.0426386892,   0.0424714797,   0.0423021802, & 
     0.0421100000,   0.0418798428,   0.0416193893,   0.0413420144,   0.0410610930, & 
     0.0407900000,   0.0405391497,   0.0403071136,   0.0400895027,   0.0398819278, & 
     0.0396800000,   0.0394798785,   0.0392799164,   0.0390790149,   0.0388760757, & 
     0.0386700000,   0.0384602963,   0.0382489010,   0.0380383576,   0.0378312094, & 
     0.0376300000,   0.0374358963,   0.0372445596,   0.0370502748,   0.0368473266, & 
     0.0366300000,   0.0363952385,   0.0361506205,   0.0359063833,   0.0356727641, & 
     0.0354600000,   0.0352752298,   0.0351131983,   0.0349655519,   0.0348239370, & 
     0.0346800000,   0.0345271224,   0.0343656264,   0.0341975692,   0.0340250079, & 
     0.0338500000,   0.0336742805,   0.0334982961,   0.0333221715,   0.0331460312, & 
     0.0329700000,   0.0327941556,   0.0326183892,   0.0324425449,   0.0322664671, & 
     0.0320900000,   0.0319129369,   0.0317348671,   0.0315553289,   0.0313738604, & 
     0.0311900000,   0.0310040966,   0.0308197422,   0.0306413395,   0.0304732912, & 
     0.0303200000,   0.0301839566,   0.0300600040,   0.0299410730,   0.0298200947, & 
     0.0296900000,   0.0295456769,   0.0293898419,   0.0292271684,   0.0290623300, & 
     0.0289000000,   0.0287438956,   0.0285939083,   0.0284489732,   0.0283080254, & 
     0.0281700000,   0.0280339406,   0.0278993248,   0.0277657387,   0.0276327684, & 
     0.0275000000,   0.0273669819,   0.0272331124,   0.0270977519,   0.0269602609, & 
     0.0268200000,   0.0266768516,   0.0265327856,   0.0263902938,   0.0262518680, & 
     0.0261200000,   0.0259957717,   0.0258746252,   0.0257505930,   0.0256177071, & 
     0.0254700000,   0.0253043017,   0.0251286334,   0.0249538143,   0.0247906635, & 
     0.0246500000,   0.0245391016,   0.0244510811,   0.0243755097,   0.0243019589, & 
     0.0242200000,   0.0241217718,   0.0240096823,   0.0238887070,   0.0237638211, & 
     0.0236400000,   0.0235210111,   0.0234057896,   0.0232920624,   0.0231775569, & 
     0.0230600000,   0.0229380237,   0.0228138794,   0.0226907233,   0.0225717114, & 
     0.0224600000,   0.0223576139,   0.0222620527,   0.0221696844,   0.0220768775, & 
     0.0219800000,   0.0218764806,   0.0217679899,   0.0216572589,   0.0215470186, & 
     0.0214400000,   0.0213382236,   0.0212408676,   0.0211463999,   0.0210532881, & 
     0.0209600000,   0.0208653451,   0.0207694996,   0.0206729816,   0.0205763091, & 
     0.0204800000,   0.0203843961,   0.0202891338,   0.0201936736,   0.0200974755, & 
     0.0200000000,   0.0199009907,   0.0198013251,   0.0197021641,   0.0196046688, & 
     0.0195100000,   0.0194189213,   0.0193306059,   0.0192438300,   0.0191573694, & 
     0.0190700000,   0.0189805243,   0.0188878511,   0.0187909158,   0.0186886536, & 
     0.0185800000,   0.0184648214,   0.0183467095,   0.0182301867,   0.0181197760, & 
     0.0180200000,   0.0179341099,   0.0178602710,   0.0177953772,   0.0177363222, & 
     0.0176800000,   0.0176236190,   0.0175656465,   0.0175048645,   0.0174400550, & 
     0.0173700000,   0.0172940542,   0.0172138631,   0.0171316449,   0.0170496178, & 
     0.0169700000,   0.0168942442,   0.0168207411,   0.0167471159,   0.0166709938, & 
     0.0165900000,   0.0165027289,   0.0164116524,   0.0163202114,   0.0162318469, & 
     0.0161500000,   0.0160771600,   0.0160120093,   0.0159522786,   0.0158956986, & 
     0.0158400000,   0.0157832710,   0.0157250303,   0.0156651541,   0.0156035186, & 
     0.0155400000,   0.0154745559,   0.0154074694,   0.0153391049,   0.0152698270, & 
     0.0152000000,   0.0151300254,   0.0150604522,   0.0149918662,   0.0149248535, & 
     0.0148600000,   0.0147976625,   0.0147372820,   0.0146780702,   0.0146192389, & 
     0.0145600000,   0.0144997245,   0.0144384199,   0.0143762531,   0.0143133909, & 
     0.0142500000,   0.0141862396,   0.0141222384,   0.0140581174,   0.0139939976, & 
     0.0139300000,   0.0138662770,   0.0138031064,   0.0137407972,   0.0136796587, & 
     0.0136200000,   0.0135620124,   0.0135054161,   0.0134498136,   0.0133948074, & 
     0.0133400000,   0.0132851135,   0.0132303493,   0.0131760283,   0.0131224715, & 
     0.0130700000,   0.0130187335,   0.0129679867,   0.0129168732,   0.0128645065, & 
     0.0128100000,   0.0127527525,   0.0126933038,   0.0126324788,   0.0125711025, & 
     0.0125100000,   0.0124498564,   0.0123907981,   0.0123328115,   0.0122758833, & 
     0.0122200000,   0.0121649418,   0.0121096639,   0.0120529150,   0.0119934441, & 
     0.0119300000,   0.0118622164,   0.0117932665,   0.0117272084,   0.0116681002, & 
     0.0116200000,   0.0115850727,   0.0115579103,   0.0115312115,   0.0114976751, & 
     0.0114500000,   0.0113835728,   0.0113045325,   0.0112217057,   0.0111439193, & 
     0.0110800000,   0.0110362359,   0.0110087599,   0.0109911658,   0.0109770479, & 
     0.0109600000,   0.0109350035,   0.0109025881,   0.0108646710,   0.0108231693, & 
     0.0107800000,   0.0107367903,   0.0106940077,   0.0106518300,   0.0106104349, & 
     0.0105700000,   0.0105306355,   0.0104921811,   0.0104544090,   0.0104170912, & 
     0.0103800000,   0.0103429077,   0.0103055878,   0.0102678140,   0.0102293602, &
     0.0101900000,   0.0101495978,   0.0101083798/)
     
     

! specific absorption coefficients of phytoplankton: applicable: 300 - 398nm; unit: m ** 2 / mg
! power-law: a*p(wl) = A(wl) * chl_con ** (B(wl))
! 'specific_A': A(wl); 'specific_B': B(wl)

     real(8), parameter, dimension(913)::specific_A = (/0.0000000000, &
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0736000000,   0.0755000000,   0.0766000000,   0.0768000000,   0.0763000000, & 
     0.0755000000,   0.0738000000,   0.0712000000,   0.0683000000,   0.0650000000, & 
     0.0606000000,   0.0547000000,   0.0481000000,   0.0427000000,   0.0390000000, & 
     0.0371000000,   0.0367000000,   0.0372000000,   0.0378000000,   0.0391000000, & 
     0.0402000000,   0.0412000000,   0.0417000000,   0.0421000000,   0.0425000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, &
     0.0000000000,   0.0000000000,   0.0000000000/)
     

     real(8), parameter, dimension(913)::specific_B = (/0.0000000000, &
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.8524000000,   0.8507000000,   0.8483000000,   0.8423000000,   0.8318000000, & 
     0.8198000000,   0.8078000000,   0.7973000000,   0.7898000000,   0.7852000000, & 
     0.7792000000,   0.7747000000,   0.7717000000,   0.7687000000,   0.7657000000, & 
     0.7597000000,   0.7537000000,   0.7462000000,   0.7387000000,   0.7327000000, & 
     0.7267000000,   0.7207000000,   0.7162000000,   0.7117000000,   0.7087000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, &
     0.0000000000,   0.0000000000,   0.0000000000/)
     

! specific absorption coefficients of phytoplankton: applicable: 400 - 700nm; unit: m ** 2 / mg
! size-fraction phytoplankton absoption coefficients
! three size: pico- (2nm<), nano- (2-20nm), micro-phytoplankton (>20nm)
! 'specific_ap'; 'specific_an'; 'specific_am';

     real(8), parameter, dimension(913)::specific_ap = (/0.0000000000, &
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0600000000,   0.0766859923,   0.0883036734,   0.0969688531,   0.1039074604, & 
     0.1100000000,   0.1164734905,   0.1234332505,   0.1300671573,   0.1355630882, & 
     0.1391089205,   0.1398561145,   0.1369533367,   0.1319123476,   0.1267319788, & 
     0.1207950989,   0.1144804924,   0.1087926655,   0.1034642935,   0.0982764908, & 
     0.0930317601,   0.0875326044,   0.0815815265,   0.0747781821,   0.0668734891, & 
     0.0585087391,   0.0503568289,   0.0430906553,   0.0371995626,   0.0321207245, & 
     0.0280000000,   0.0243618844,   0.0215772494,   0.0208236359,   0.0206129922, & 
     0.0205049612,   0.0203953425,   0.0201799362,   0.0194122105,   0.0175908571, & 
     0.0160000000,   0.0147547429,   0.0139281924,   0.0127904202,   0.0105558527, & 
     0.0077945138,   0.0051789305,   0.0033816298,   0.0028523562,   0.0026470190, & 
     0.0025000000,   0.0023891924,   0.0023163336,   0.0022432507,   0.0021317710, & 
     0.0018817884,   0.0009741355,   0.0019045203,   0.0034258619,   0.0046439781, & 
     0.0057933219,   0.0068451382,   0.0077706722,   0.0085411688,   0.0091177771, & 
     0.0094032508,   0.0098125635,   0.0154661541,   0.0233215962,   0.0275000000, & 
     0.0179083029,   0.0089774362,   0.0053186048,   0.0024840118,   0.0006522721, & 
     0.0000020000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, &
     0.0000000000,   0.0000000000,   0.0000000000/)
     
     
     real(8), parameter, dimension(913)::specific_an = (/0.0000000000, &
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0450000000,   0.0521267539,   0.0603265568,   0.0674234856,   0.0723917960, & 
     0.0760000000,   0.0788202292,   0.0814822488,   0.0838070480,   0.0856156164, & 
     0.0867289432,   0.0869033053,   0.0850197567,   0.0820002907,   0.0794876634, & 
     0.0777007519,   0.0756568779,   0.0722696408,   0.0670123698,   0.0605624118, & 
     0.0536343945,   0.0469429458,   0.0412026933,   0.0364769671,   0.0317240061, & 
     0.0272674768,   0.0235000585,   0.0208144304,   0.0194631935,   0.0186973951, & 
     0.0180000000,   0.0171409231,   0.0163212308,   0.0157496923,   0.0153709231, & 
     0.0150705128,   0.0147475385,   0.0143010769,   0.0134240000,   0.0123680000, & 
     0.0120000000,   0.0116320000,   0.0106960000,   0.0092507273,   0.0073950909, & 
     0.0053712121,   0.0035485455,   0.0022965455,   0.0018546732,   0.0016487568, & 
     0.0015000000,   0.0013739898,   0.0012717390,   0.0011793994,   0.0010831224, & 
     0.0009643352,   0.0008106448,   0.0012455180,   0.0020898665,   0.0029076930, & 
     0.0037620761,   0.0046359338,   0.0055121841,   0.0063737450,   0.0071885104, & 
     0.0077906982,   0.0086607923,   0.0120752128,   0.0187648713,   0.0248000000, & 
     0.0161379046,   0.0080790041,   0.0047846632,   0.0022336441,   0.0005858039, & 
     0.0000010000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, &
     0.0000000000,   0.0000000000,   0.0000000000/)

     real(8), parameter, dimension(913)::specific_am = (/0.0000000000, &
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0580000000,   0.0599437409,   0.0625517241,   0.0649990318,   0.0668335767, & 
     0.0680000000,   0.0686122486,   0.0691386569,   0.0695646867,   0.0698757996, & 
     0.0700574574,   0.0699371128,   0.0668583599,   0.0622880369,   0.0593002674, & 
     0.0576439322,   0.0563153638,   0.0551271311,   0.0541404640,   0.0532574216, & 
     0.0523701768,   0.0513709025,   0.0501517714,   0.0485865230,   0.0466112868, & 
     0.0442887697,   0.0416855434,   0.0388681797,   0.0357188960,   0.0317222759, & 
     0.0281000000,   0.0252717862,   0.0229051868,   0.0212290467,   0.0198175473, & 
     0.0185960146,   0.0175837572,   0.0168000838,   0.0162548432,   0.0159426286, & 
     0.0158000000,   0.0163525333,   0.0167859600,   0.0173059257,   0.0181409465, & 
     0.0191734123,   0.0202831862,   0.0213501312,   0.0222657888,   0.0230908679, & 
     0.0240000000,   0.0251331876,   0.0264386872,   0.0278041413,   0.0291171919, & 
     0.0302814938,   0.0313289483,   0.0318916923,   0.0319636480,   0.0317096960, & 
     0.0312960000,   0.0308208640,   0.0303825920,   0.0300794880,   0.0301594792, & 
     0.0332149891,   0.0379153129,   0.0425343816,   0.0470339044,   0.0500000000, & 
     0.0475112313,   0.0427547436,   0.0363774432,   0.0278683411,   0.0173637544, & 
     0.0050000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, & 
     0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000,   0.0000000000, &
     0.0000000000,   0.0000000000,   0.0000000000/)

end module rrsw_ocaux


